@@ -1,15 +1,19 @@
 # -*- test-case-name: foolscap.test.test_pb -*-
 
-import os.path, weakref
+import os.path, weakref, binascii
 from zope.interface import implements
-from twisted.internet import defer, protocol
+from twisted.internet import defer, protocol, error
 from twisted.application import service, strports
-from twisted.python import log
+from twisted.python.failure import Failure
 
 from foolscap import ipb, base32, negotiate, broker, observer, eventual, storage
 from foolscap.referenceable import SturdyRef
 from foolscap.tokens import PBError, BananaError, WrongTubIdError
 from foolscap.reconnector import Reconnector
+from foolscap.logging import log as flog
+from foolscap.logging import log
+from foolscap.logging import publish as flog_publish
+from foolscap.logging.log import WEIRD, UNUSUAL
 
 crypto_available = False
 try:
@@ -29,6 +33,8 @@ class Listener(protocol.ServerFactory):
     Unauthenticated Tubs use a TubID of 'None'. There may be at most one such
     Tub attached to any given Listener."""
 
+    noisy = False
+
     # this also serves as the ServerFactory
 
     def __init__(self, port, options={},
@@ -117,10 +123,20 @@ class Listener(protocol.ServerFactory):
     def removeRedirect(self, tubID):
         del self.redirects[tubID]
 
+    def startFactory(self):
+        log.msg("Starting factory %r" % self)
+        return protocol.ServerFactory.startFactory(self)
+    def stopFactory(self):
+        log.msg("Stopping factory %r" % self)
+        return protocol.ServerFactory.stopFactory(self)
+
+
     def buildProtocol(self, addr):
         """Return a Broker attached to me (as the service provider).
         """
-        proto = self.negotiationClass()
+        lp = log.msg("%s accepting connection from %s" % (self, addr),
+                     addr=(addr.host, addr.port))
+        proto = self.negotiationClass(logparent=lp)
         proto.initServer(self)
         proto.factory = self
         return proto
@@ -194,6 +210,7 @@ class Tub(service.MultiService):
     brokerClass = broker.Broker
     keepaliveTimeout = 4*60 # ping when connection has been idle this long
     disconnectTimeout = None # disconnect after this much idle time
+    tubID = None
 
     def __init__(self, certData=None, certFile=None, options={}):
         service.MultiService.__init__(self)
@@ -203,6 +220,9 @@ class Tub(service.MultiService):
         else:
             self.setupEncryption(certData)
 
+    def __repr__(self):
+        return "<Tub id=%s>" % self.tubID
+
     def setupEncryptionFile(self, certFile):
         try:
             certData = open(certFile, "rb").read()
@@ -227,11 +247,34 @@ class Tub(service.MultiService):
         self.myCertificate = cert
         self.tubID = crypto.digest32(cert.digest("sha1"))
 
+    def make_incarnation(self):
+        unique = os.urandom(8)
+        # TODO: it'd be nice to have a sequential component, so incarnations
+        # could be ordered, but it requires disk space
+        sequential = None
+        self.incarnation = (unique, sequential)
+        self.incarnation_string = binascii.b2a_hex(unique)
+
+    def getIncarnationString(self):
+        return self.incarnation_string
+
     def setup(self, options):
         self.options = options
         self.listeners = []
         self.locationHints = []
 
+        # duplicate-connection management
+        self.make_incarnation()
+
+        # the master_table records the master-seqnum we used for the last
+        # established connection with the given tubid. It only contains
+        # entries for which we were the master.
+        self.master_table = {} # k:tubid, v:seqnum
+        # the slave_table records the (master-IR,master-seqnum) pair for the
+        # last established conenction with the given tubid. It only contains
+        # entries for which we were the slave.
+        self.slave_table = {} # k:tubid, v:(master-IR,seqnum)
+
         # local Referenceables
         self.nameToReference = weakref.WeakValueDictionary()
         self.referenceToName = weakref.WeakKeyDictionary()
@@ -252,6 +295,15 @@ class Tub(service.MultiService):
 
         self._pending_getReferences = [] # list of (d, furl) pairs
 
+        self._logport = None
+        self._logport_furl = None
+        self._logport_furlfile = None
+
+        self._log_gatherer_furl = None
+        self._log_gatherer_furlfile = None
+
+        self._handle_old_duplicate_connections = False
+
     def setOption(self, name, value):
         if name == "logLocalFailures":
             # log (with log.err) any exceptions that occur during the
@@ -277,9 +329,91 @@ class Tub(service.MultiService):
             self.keepaliveTimeout = value
         elif name == "disconnectTimeout":
             self.disconnectTimeout = value
+        elif name == "logport-furlfile":
+            self.setLogPortFURLFile(value)
+        elif name == "log-gatherer-furl":
+            self.setLogGathererFURL(value)
+        elif name == "log-gatherer-furlfile":
+            self.setLogGathererFURLFile(value)
+        elif name == "bridge-twisted-logs":
+            if value:
+                tlb = flog.TwistedLogBridge(self.tubID)
+                flog.setTwistedLogBridge(tlb)
+            else:
+                flog.setTwistedLogBridge(None)
+        elif name == "handle-old-duplicate-connections":
+            if value is True:
+                value = 60
+            self._handle_old_duplicate_connections = int(value)
         else:
             raise KeyError("unknown option name '%s'" % name)
 
+    def setLogGathererFURL(self, gatherer_furl):
+        assert not self._log_gatherer_furl and not self._log_gatherer_furlfile
+        self._log_gatherer_furl = gatherer_furl
+        self._maybeConnectToGatherer()
+
+    def setLogGathererFURLFile(self, gatherer_furlfile):
+        assert not self._log_gatherer_furl and not self._log_gatherer_furlfile
+        self._log_gatherer_furlfile = gatherer_furlfile
+        self._maybeConnectToGatherer()
+
+    def _maybeConnectToGatherer(self):
+        furl = self._log_gatherer_furl
+        if not furl and self._log_gatherer_furlfile:
+            try:
+                furl = open(self._log_gatherer_furlfile, "r").read().strip()
+            except EnvironmentError:
+                pass
+        if furl:
+            connector = self.connectTo(furl, self._log_gatherer_connected)
+
+    def _log_gatherer_connected(self, rref):
+        # we want the logport's furl to be nailed down now, so we'll use the
+        # right (persistent) name even if the user never calls
+        # tub.getLogPortFURL() directly.
+        ignored = self.getLogPortFURL()
+        tubID = self.tubID
+        if tubID is None:
+            # RILogGatherer.logport requires a string for nodeid=
+            tubID = '<unauth>'
+        rref.callRemote('logport', tubID, self.getLogPort())
+
+
+    def getLogPort(self):
+        return self._maybeCreateLogPort()
+
+    def _maybeCreateLogPort(self):
+        if not self._logport:
+            self._logport = flog_publish.LogPublisher(flog.theLogger)
+        return self._logport
+
+    def setLogPortFURLFile(self, furlfile):
+        self._logport_furlfile = furlfile
+        self._maybeCreateLogPortFURLFile()
+
+    def _maybeCreateLogPortFURLFile(self):
+        if not self._logport_furlfile:
+            return
+        if not self.locationHints:
+            return
+        # getLogPortFURL() creates the logport-furlfile as a side-effect
+        ignored = self.getLogPortFURL()
+
+    def getLogPortFURL(self):
+        if self._logport_furl:
+            return self._logport_furl
+        furlfile = self._logport_furlfile
+        # the Tub must be running and configured (setLocation) by now
+        self._logport_furl = self.registerReference(self.getLogPort(),
+                                                    furlFile=furlfile)
+        return self._logport_furl
+
+
+    def log(self, *args, **kwargs):
+        kwargs['tubID'] = self.tubID
+        return log.msg(*args, **kwargs)
+
     def createCertificate(self):
         # this is copied from test_sslverify.py
         dn = crypto.DistinguishedName(commonName="newpb_thingy")
@@ -319,6 +453,7 @@ class Tub(service.MultiService):
             raise PBError("Unauthenticated tubs may only have one "
                           "location hint")
         self.locationHints = hints
+        self._maybeCreateLogPortFURLFile()
 
     def listenOn(self, what, options={}):
         """Start listening for connections.
@@ -384,8 +519,8 @@ class Tub(service.MultiService):
             # which never shut down a Tub aren't going to care), and it is
             # more important to let application code run normally than to
             # force an error here.
-            log.msg("Tub.connectorFinished: WEIRD, %s is not in %s"
-                    % (c, self._activeConnectors))
+            self.log("Tub.connectorFinished: WEIRD, %s is not in %s"
+                     % (c, self._activeConnectors), level=WEIRD)
             return
         self._activeConnectors.remove(c)
         if not self.running and not self._activeConnectors:
@@ -434,10 +569,11 @@ class Tub(service.MultiService):
 
         if self.brokers or self.unauthenticatedBrokers:
             dl.append(self._allBrokersAreDisconnected.whenFired())
+        why = Failure(error.ConnectionDone("Tub.stopService was called"))
         for b in self.brokers.values():
-            b.shutdown()
+            b.shutdown(why, fireDisconnectWatchers=False)
         for b in self.unauthenticatedBrokers:
-            b.shutdown()
+            b.shutdown(why, fireDisconnectWatchers=False)
 
         return defer.DeferredList(dl)
 
@@ -500,7 +636,10 @@ class Tub(service.MultiService):
             self.strongReferences.append(ref)
         furl = self.buildURL(name)
         if furlFile:
+            need_to_chmod = not os.path.exists(furlFile)
             open(furlFile, "w").write(furl + "\n")
+            if need_to_chmod:
+                os.chmod(furlFile, 0600)
         return furl
 
     # this is called by either registerReference or by
@@ -689,8 +828,8 @@ class Tub(service.MultiService):
         if self.running:
             rc.startConnecting(self)
         else:
-            log.msg("Tub.connectTo(%s) queued until Tub.startService called"
-                    % _sturdyOrURL)
+            self.log("Tub.connectTo(%s) queued until Tub.startService called"
+                     % _sturdyOrURL, level=UNUSUAL)
         self.reconnectors.append(rc)
         return rc
 
@@ -775,6 +914,7 @@ class Tub(service.MultiService):
                 d.errback(why)
 
     def brokerAttached(self, tubref, broker, isClient):
+        assert self.running
         if not tubref:
             # this is an inbound connection from an unauthenticated Tub
             assert not isClient
@@ -794,19 +934,24 @@ class Tub(service.MultiService):
             del self.tubConnectors[tubref]
 
         if tubref in self.brokers:
-            # oops, this shouldn't happen but it isn't fatal. Raise
-            # BananaError so the Negotiation will drop the connection
+            # this shouldn't happen: acceptDecision is supposed to drop any
+            # existing old connection first.
+            self.log("ERROR: unexpected duplicate connection from %s" % tubref)
             raise BananaError("unexpected duplicate connection")
         self.brokers[tubref] = broker
 
         # now inform everyone who's been waiting on it
         if tubref in self.waitingForBrokers:
-            waiting = self.waitingForBrokers[tubref]
+            for d in self.waitingForBrokers[tubref]:
+                eventual.eventually(d.callback, broker)
             del self.waitingForBrokers[tubref]
-            for d in waiting:
-                d.callback(broker)
 
     def brokerDetached(self, broker, why):
+        # a loopback connection will produce two Brokers that both use the
+        # same tubref. Both will shut down about the same time. Make sure
+        # this doesn't confuse us.
+        had_connections = (bool(self.brokers) or
+                           bool(self.unauthenticatedBrokers))
         # the Broker will have already severed all active references
         for tubref in self.brokers.keys():
             if self.brokers[tubref] is broker:
@@ -814,8 +959,12 @@ class Tub(service.MultiService):
         if broker in self.unauthenticatedBrokers:
             self.unauthenticatedBrokers.remove(broker)
         # if the Tub has already shut down, we may need to notify observers
-        # who are waiting for all of our connections to finish shutting down
+        # who are waiting for all of our connections to finish shutting down.
+        # Only do this if we actually transitioned from having some
+        # connections to not having any connections.
+
         if (not self.running
+            and had_connections
             and not self.brokers
             and not self.unauthenticatedBrokers):
             self._allBrokersAreDisconnected.fire(self)
