@@ -22,10 +22,26 @@ URIExtensionData = StringConstraint(1000)
 LeaseRenewSecret = Hash # used to protect bucket lease renewal requests
 LeaseCancelSecret = Hash # used to protect bucket lease cancellation requests
 
+# Announcements are (FURL, service_name, remoteinterface_name,
+#                    nickname, my_version, oldest_supported)
+#  the (FURL, service_name, remoteinterface_name) refer to the service being
+#  announced. The (nickname, my_version, oldest_supported) refer to the
+#  client as a whole. The my_version/oldest_supported strings can be parsed
+#  by an allmydata.util.version.Version instance, and then compared. The
+#  first goal is to make sure that nodes are not confused by speaking to an
+#  incompatible peer. The second goal is to enable the development of
+#  backwards-compatibility code.
 
-class RIIntroducerClient(RemoteInterface):
-    def new_peers(furls=SetOf(FURL)):
+Announcement = TupleOf(FURL, str, str,
+                       str, str, str)
+
+class RIIntroducerSubscriberClient(RemoteInterface):
+    __remote_name__ = "RIIntroducerSubscriberClient.tahoe.allmydata.com"
+
+    def announce(announcements=SetOf(Announcement)):
+        """I accept announcements from the publisher."""
         return None
+
     def set_encoding_parameters(parameters=(int, int, int)):
         """Advise the client of the recommended k-of-n encoding parameters
         for this grid. 'parameters' is a tuple of (k, desired, n), where 'n'
@@ -43,28 +59,104 @@ class RIIntroducerClient(RemoteInterface):
         """
         return None
 
-class RIIntroducer(RemoteInterface):
-    def hello(node=RIIntroducerClient, furl=FURL):
+# When Foolscap can handle multiple interfaces (Foolscap#17), the
+# full-powered introducer will implement both RIIntroducerPublisher and
+# RIIntroducerSubscriberService. Until then, we define
+# RIIntroducerPublisherAndSubscriberService as a combination of the two, and
+# make everybody use that.
+
+class RIIntroducerPublisher(RemoteInterface):
+    """To publish a service to the world, connect to me and give me your
+    announcement message. I will deliver a copy to all connected subscribers."""
+    __remote_name__ = "RIIntroducerPublisher.tahoe.allmydata.com"
+
+    def publish(announcement=Announcement):
+        # canary?
         return None
 
-class RIClient(RemoteInterface):
-    def get_versions():
-        """Return a tuple of (my_version, oldest_supported) strings.
+class RIIntroducerSubscriberService(RemoteInterface):
+    __remote_name__ = "RIIntroducerSubscriberService.tahoe.allmydata.com"
 
-        Each string can be parsed by an allmydata.util.version.Version
-        instance, and then compared. The first goal is to make sure that
-        nodes are not confused by speaking to an incompatible peer. The
-        second goal is to enable the development of backwards-compatibility
-        code.
-
-        This method is likely to change in incompatible ways until we get the
-        whole compatibility scheme nailed down.
+    def subscribe(subscriber=RIIntroducerSubscriberClient, service_name=str):
+        """Give me a subscriber reference, and I will call its new_peers()
+        method will any announcements that match the desired service name. I
+        will ignore duplicate subscriptions.
         """
-        return TupleOf(str, str)
-    def get_service(name=str):
-        return Referenceable
-    def get_nodeid():
-        return Nodeid
+        return None
+
+class RIIntroducerPublisherAndSubscriberService(RemoteInterface):
+    __remote_name__ = "RIIntroducerPublisherAndSubscriberService.tahoe.allmydata.com"
+    def publish(announcement=Announcement):
+        return None
+    def subscribe(subscriber=RIIntroducerSubscriberClient, service_name=str):
+        return None
+
+class IIntroducerClient(Interface):
+    """I provide service introduction facilities for a node. I help nodes
+    publish their services to the rest of the world, and I help them learn
+    about services available on other nodes."""
+
+    def publish(furl, service_name, remoteinterface_name):
+        """Once you call this, I will tell the world that the Referenceable
+        available at FURL is available to provide a service named
+        SERVICE_NAME. The precise definition of the service being provided is
+        identified by the Foolscap 'remote interface name' in the last
+        parameter: this is supposed to be a globally-unique string that
+        identifies the RemoteInterface that is implemented."""
+
+    def subscribe_to(service_name):
+        """Call this if you will eventually want to use services with the
+        given SERVICE_NAME. This will prompt me to subscribe to announcements
+        of those services. You can pick up the announcements later by calling
+        get_all_connections_for() or get_permuted_peers().
+        """
+
+    def get_all_connections():
+        """Return a frozenset of (nodeid, service_name, rref) tuples, one for
+        each active connection we've established to a remote service. This is
+        mostly useful for unit tests that need to wait until a certain number
+        of connections have been made."""
+
+    def get_all_connectors():
+        """Return a dict that maps from (nodeid, service_name) to a
+        RemoteServiceConnector instance for all services that we are actively
+        trying to connect to. Each RemoteServiceConnector has the following
+        public attributes::
+
+          service_name: the type of service provided, like 'storage'
+          announcement_time: when we first heard about this service
+          last_connect_time: when we last established a connection
+          last_loss_time: when we last lost a connection
+
+          version: the peer's version, from the most recent connection
+          oldest_supported: the peer's oldest supported version, same
+
+          rref: the RemoteReference, if connected, otherwise None
+          remote_host: the IAddress, if connected, otherwise None
+
+        This method is intended for monitoring interfaces, such as a web page
+        which describes connecting and connected peers.
+        """
+
+    def get_all_peerids():
+        """Return a frozenset of all peerids to whom we have a connection (to
+        one or more services) established. Mostly useful for unit tests."""
+
+    def get_all_connections_for(service_name):
+        """Return a frozenset of (nodeid, service_name, rref) tuples, one
+        for each active connection that provides the given SERVICE_NAME."""
+
+    def get_permuted_peers(service_name, key):
+        """Returns an ordered list of (peerid, rref) tuples, selecting from
+        the connections that provide SERVICE_NAME, using a hash-based
+        permutation keyed by KEY. This randomizes the service list in a
+        repeatable way, to distribute load over many peers.
+        """
+
+    def connected_to_introducer():
+        """Returns a boolean, True if we are currently connected to the
+        introducer, False if not."""
+
 
 class RIBucketWriter(RemoteInterface):
     def write(offset=int, data=ShareData):
@@ -78,6 +170,11 @@ class RIBucketWriter(RemoteInterface):
         """
         return None
 
+    def abort():
+        """Abandon all the data that has been written.
+        """
+        return None
+
 class RIBucketReader(RemoteInterface):
     def read(offset=int, length=int):
         return ShareData
@@ -98,6 +195,21 @@ ReadData = ListOf(ShareData)
 # returns data[offset:offset+length] for each element of TestVector
 
 class RIStorageServer(RemoteInterface):
+    __remote_name__ = "RIStorageServer.tahoe.allmydata.com"
+
+    def get_versions():
+        """Return a tuple of (my_version, oldest_supported) strings.
+        Each string can be parsed by an allmydata.util.version.Version
+        instance, and then compared. The first goal is to make sure that
+        nodes are not confused by speaking to an incompatible peer. The
+        second goal is to enable the development of backwards-compatibility
+        code.
+
+        This method is likely to change in incompatible ways until we get the
+        whole compatibility scheme nailed down.
+        """
+        return TupleOf(str, str)
+
     def allocate_buckets(storage_index=StorageIndex,
                          renew_secret=LeaseRenewSecret,
                          cancel_secret=LeaseCancelSecret,
@@ -374,7 +486,6 @@ class IFileURI(Interface):
 
 class IMutableFileURI(Interface):
     """I am a URI which represents a mutable filenode."""
-    pass
 class INewDirectoryURI(Interface):
     pass
 class IReadonlyNewDirectoryURI(Interface):
@@ -452,7 +563,7 @@ class IMutableFileNode(IFileNode, IMutableFilesystemNode):
         more advanced API will signal and provide access to the multiple
         heads."""
 
-    def replace(newdata, wait_for_numpeers=None):
+    def replace(newdata):
         """Replace the old contents with the new data. Returns a Deferred
         that fires (with None) when the operation is complete.
 
@@ -476,6 +587,11 @@ class IMutableFileNode(IFileNode, IMutableFilesystemNode):
         """
 
 class IDirectoryNode(IMutableFilesystemNode):
+    """I represent a name-to-child mapping, holding the tahoe equivalent of a
+    directory. All child names are unicode strings, and all children are some
+    sort of IFilesystemNode (either files or subdirectories).
+    """
+
     def get_uri():
         """
         The dirnode ('1') URI returned by this method can be used in
@@ -496,17 +612,33 @@ class IDirectoryNode(IMutableFilesystemNode):
 
     def list():
         """I return a Deferred that fires with a dictionary mapping child
-        name to (node, metadata_dict) tuples, in which 'node' is either an
-        IFileNode or IDirectoryNode, and 'metadata_dict' is a dictionary of
-        metadata."""
+        name (a unicode string) to (node, metadata_dict) tuples, in which
+        'node' is either an IFileNode or IDirectoryNode, and 'metadata_dict'
+        is a dictionary of metadata."""
 
     def has_child(name):
         """I return a Deferred that fires with a boolean, True if there
-        exists a child of the given name, False if not."""
+        exists a child of the given name, False if not. The child name must
+        be a unicode string."""
 
     def get(name):
         """I return a Deferred that fires with a specific named child node,
-        either an IFileNode or an IDirectoryNode."""
+        either an IFileNode or an IDirectoryNode. The child name must be a
+        unicode string."""
+
+    def get_metadata_for(name):
+        """I return a Deferred that fires with the metadata dictionary for a
+        specific named child node. This metadata is stored in the *edge*, not
+        in the child, so it is attached to the parent dirnode rather than the
+        child dir-or-file-node. The child name must be a unicode string."""
+
+    def set_metadata_for(name, metadata):
+        """I replace any existing metadata for the named child with the new
+        metadata. The child name must be a unicode string. This metadata is
+        stored in the *edge*, not in the child, so it is attached to the
+        parent dirnode rather than the child dir-or-file-node. I return a
+        Deferred (that fires with this dirnode) when the operation is
+        complete."""
 
     def get_child_at_path(path):
         """Transform a child path into an IDirectoryNode or IFileNode.
@@ -516,62 +648,84 @@ class IDirectoryNode(IMutableFilesystemNode):
         errbacks with IndexError if the node could not be found.
 
         The path can be either a single string (slash-separated) or a list of
-        path-name elements.
+        path-name elements. All elements must be unicode strings.
         """
 
-    def set_uri(name, child_uri):
+    def set_uri(name, child_uri, metadata=None):
         """I add a child (by URI) at the specific name. I return a Deferred
         that fires when the operation finishes. I will replace any existing
-        child of the same name.
+        child of the same name. The child name must be a unicode string.
 
         The child_uri could be for a file, or for a directory (either
         read-write or read-only, using a URI that came from get_uri() ).
 
+        If metadata= is provided, I will use it as the metadata for the named
+        edge. This will replace any existing metadata. If metadata= is left
+        as the default value of None, I will set ['mtime'] to the current
+        time, and I will set ['ctime'] to the current time if there was not
+        already a child by this name present. This roughly matches the
+        ctime/mtime semantics of traditional filesystems.
+
         If this directory node is read-only, the Deferred will errback with a
         NotMutableError."""
 
     def set_uris(entries):
-        """Add multiple (name, child_uri) pairs to a directory node. Returns
-        a Deferred that fires (with None) when the operation finishes. This
-        is equivalent to calling set_uri() multiple times, but is much more
-        efficient.
+        """Add multiple (name, child_uri) pairs (or (name, child_uri,
+        metadata) triples) to a directory node. Returns a Deferred that fires
+        (with None) when the operation finishes. This is equivalent to
+        calling set_uri() multiple times, but is much more efficient. All
+        child names must be unicode strings.
         """
 
-    def set_node(name, child):
+    def set_node(name, child, metadata=None):
         """I add a child at the specific name. I return a Deferred that fires
         when the operation finishes. This Deferred will fire with the child
         node that was just added. I will replace any existing child of the
-        same name.
+        same name. The child name must be a unicode string.
+
+        If metadata= is provided, I will use it as the metadata for the named
+        edge. This will replace any existing metadata. If metadata= is left
+        as the default value of None, I will set ['mtime'] to the current
+        time, and I will set ['ctime'] to the current time if there was not
+        already a child by this name present. This roughly matches the
+        ctime/mtime semantics of traditional filesystems.
 
         If this directory node is read-only, the Deferred will errback with a
         NotMutableError."""
 
     def set_nodes(entries):
-        """Add multiple (name, child_node) pairs to a directory node. Returns
-        a Deferred that fires (with None) when the operation finishes. This
-        is equivalent to calling set_node() multiple times, but is much more
-        efficient."""
+        """Add multiple (name, child_node) pairs (or (name, child_node,
+        metadata) triples) to a directory node. Returns a Deferred that fires
+        (with None) when the operation finishes. This is equivalent to
+        calling set_node() multiple times, but is much more efficient. All
+        child names must be unicode strings."""
 
 
-    def add_file(name, uploadable):
+    def add_file(name, uploadable, metadata=None):
         """I upload a file (using the given IUploadable), then attach the
-        resulting FileNode to the directory at the given name. I return a
-        Deferred that fires (with the IFileNode of the uploaded file) when
-        the operation completes."""
+        resulting FileNode to the directory at the given name. I set metadata
+        the same way as set_uri and set_node. The child name must be a
+        unicode string.
+
+        I return a Deferred that fires (with the IFileNode of the uploaded
+        file) when the operation completes."""
 
     def delete(name):
         """I remove the child at the specific name. I return a Deferred that
-        fires when the operation finishes."""
+        fires when the operation finishes. The child name must be a unicode
+        string."""
 
     def create_empty_directory(name):
-        """I create and attach an empty directory at the given name. I return
-        a Deferred that fires when the operation finishes."""
+        """I create and attach an empty directory at the given name. The
+        child name must be a unicode string. I return a Deferred that fires
+        when the operation finishes."""
 
     def move_child_to(current_child_name, new_parent, new_child_name=None):
         """I take one of my children and move them to a new parent. The child
         is referenced by name. On the new parent, the child will live under
-        'new_child_name', which defaults to 'current_child_name'. I return a
-        Deferred that fires when the operation finishes."""
+        'new_child_name', which defaults to 'current_child_name'. TODO: what
+        should we do about metadata? I return a Deferred that fires when the
+        operation finishes. The child name must be a unicode string."""
 
     def build_manifest():
         """Return a frozenset of verifier-capability strings for all nodes
@@ -963,49 +1117,61 @@ class IDownloader(Interface):
         when the download is finished, or errbacks if something went wrong."""
 
 class IEncryptedUploadable(Interface):
+    def set_upload_status(upload_status):
+        """Provide an IUploadStatus object that should be filled with status
+        information. The IEncryptedUploadable is responsible for setting
+        key-determination progress ('chk'), size, storage_index, and
+        ciphertext-fetch progress. It may delegate some of this
+        responsibility to others, in particular to the IUploadable."""
+
     def get_size():
         """This behaves just like IUploadable.get_size()."""
 
-    def set_serialized_encoding_parameters(serialized_encoding_parameters):
-        """Tell me what encoding parameters will be used for my data.
+    def get_all_encoding_parameters():
+        """Return a Deferred that fires with a tuple of
+        (k,happy,n,segment_size). The segment_size will be used as-is, and
+        must match the following constraints: it must be a multiple of k, and
+        it shouldn't be unreasonably larger than the file size (if
+        segment_size is larger than filesize, the difference must be stored
+        as padding).
 
-        'serialized_encoding_parameters' is a string which indicates how the
-        data will be encoded (codec name, blocksize, number of shares).
+        This usually passes through to the IUploadable method of the same
+        name.
 
-        I may use this when get_storage_index() is called, to influence the
-        index that I return. Or, I may just ignore it.
-
-        set_serialized_encoding_parameters() may be called 0 or 1 times. If
-        called, it must be called before get_storage_index().
+        The encoder strictly obeys the values returned by this method. To
+        make an upload use non-default encoding parameters, you must arrange
+        to control the values that this method returns.
         """
 
     def get_storage_index():
-        """Return a Deferred that fires with a 16-byte storage index. This
-        value may be influenced by the parameters earlier set by
-        set_serialized_encoding_parameters().
+        """Return a Deferred that fires with a 16-byte storage index.
         """
 
-    def set_segment_size(segment_size):
-        """Set the segment size, to allow the IEncryptedUploadable to
-        accurately create the plaintext segment hash tree. This must be
-        called before any calls to read_encrypted."""
-
-    def read_encrypted(length):
+    def read_encrypted(length, hash_only):
         """This behaves just like IUploadable.read(), but returns crypttext
-        instead of plaintext. set_segment_size() must be called before the
-        first call to read_encrypted()."""
+        instead of plaintext. If hash_only is True, then this discards the
+        data (and returns an empty list); this improves efficiency when
+        resuming an interrupted upload (where we need to compute the
+        plaintext hashes, but don't need the redundant encrypted data)."""
 
-    def get_plaintext_segment_hashtree_nodes(num_segments):
-        """Get the nodes of a merkle hash tree over the plaintext segments.
+    def get_plaintext_hashtree_leaves(first, last, num_segments):
+        """Get the leaf nodes of a merkle hash tree over the plaintext
+        segments, i.e. get the tagged hashes of the given segments. The
+        segment size is expected to be generated by the IEncryptedUploadable
+        before any plaintext is read or ciphertext produced, so that the
+        segment hashes can be generated with only a single pass.
 
-        This returns a Deferred which fires with a sequence of hashes. Each
-        hash is a node of a merkle hash tree, generally obtained from::
+        This returns a Deferred which fires with a sequence of hashes, using:
 
-         tuple(HashTree(segment_hashes))
+         tuple(segment_hashes[first:last])
 
         'num_segments' is used to assert that the number of segments that the
         IEncryptedUploadable handled matches the number of segments that the
         encoder was expecting.
+
+        This method must not be called until the final byte has been read
+        from read_encrypted(). Once this method is called, read_encrypted()
+        can never be called again.
         """
 
     def get_plaintext_hash():
@@ -1019,23 +1185,51 @@ class IEncryptedUploadable(Interface):
         """Just like IUploadable.close()."""
 
 class IUploadable(Interface):
+    def set_upload_status(upload_status):
+        """Provide an IUploadStatus object that should be filled with status
+        information. The IUploadable is responsible for setting
+        key-determination progress ('chk')."""
+
+    def set_default_encoding_parameters(params):
+        """Set the default encoding parameters, which must be a dict mapping
+        strings to ints. The meaningful keys are 'k', 'happy', 'n', and
+        'max_segment_size'. These might have an influence on the final
+        encoding parameters returned by get_all_encoding_parameters(), if the
+        Uploadable doesn't have more specific preferences.
+
+        This call is optional: if it is not used, the Uploadable will use
+        some built-in defaults. If used, this method must be called before
+        any other IUploadable methods to have any effect.
+        """
+
     def get_size():
         """Return a Deferred that will fire with the length of the data to be
         uploaded, in bytes. This will be called before the data is actually
         used, to compute encoding parameters.
         """
 
-    def set_serialized_encoding_parameters(serialized_encoding_parameters):
-        """Tell me what encoding parameters will be used for my data.
+    def get_all_encoding_parameters():
+        """Return a Deferred that fires with a tuple of
+        (k,happy,n,segment_size). The segment_size will be used as-is, and
+        must match the following constraints: it must be a multiple of k, and
+        it shouldn't be unreasonably larger than the file size (if
+        segment_size is larger than filesize, the difference must be stored
+        as padding).
 
-        'serialized_encoding_parameters' is a string which indicates how the
-        data will be encoded (codec name, blocksize, number of shares).
+        The relative values of k and n allow some IUploadables to request
+        better redundancy than others (in exchange for consuming more space
+        in the grid).
 
-        I may use this when get_encryption_key() is called, to influence the
-        key that I return. Or, I may just ignore it.
+        Larger values of segment_size reduce hash overhead, while smaller
+        values reduce memory footprint and cause data to be delivered in
+        smaller pieces (which may provide a smoother and more predictable
+        download experience).
 
-        set_serialized_encoding_parameters() may be called 0 or 1 times. If
-        called, it must be called before get_encryption_key().
+        The encoder strictly obeys the values returned by this method. To
+        make an upload use non-default encoding parameters, you must arrange
+        to control the values that this method returns. One way to influence
+        them may be to call set_encoding_parameters() before calling
+        get_all_encoding_parameters().
         """
 
     def get_encryption_key():
@@ -1082,21 +1276,22 @@ class IUploadable(Interface):
         """The upload is finished, and whatever filehandle was in use may be
         closed."""
 
+class IUploadResults(Interface):
+    """I am returned by upload() methods. I contain a number of public
+    attributes which can be read to determine the results of the upload::
+
+     .uri : the CHK read-cap for the file
+
+    """
+
 class IUploader(Interface):
-    def upload(uploadable, wait_for_numpeers=None):
+    def upload(uploadable):
         """Upload the file. 'uploadable' must impement IUploadable. This
-        returns a Deferred which fires with the URI of the file."""
+        returns a Deferred which fires with an UploadResults instance, from
+        which the URI of the file can be obtained as results.uri ."""
 
     def upload_ssk(write_capability, new_version, uploadable):
         """TODO: how should this work?"""
-    def upload_data(data):
-        """Like upload(), but accepts a string."""
-
-    def upload_filename(filename):
-        """Like upload(), but accepts an absolute pathname."""
-
-    def upload_filehandle(filehane):
-        """Like upload(), but accepts an open filehandle."""
 
 class IChecker(Interface):
     def check(uri_to_check):
@@ -1160,27 +1355,22 @@ class IChecker(Interface):
         """
 
 class IClient(Interface):
-    def upload(uploadable, wait_for_numpeers=None):
-        """Upload some data into a CHK, get back the URI string for it.
+    def upload(uploadable):
+        """Upload some data into a CHK, get back the UploadResults for it.
         @param uploadable: something that implements IUploadable
-        @param wait_for_numpeers: don't upload anything until we have at least
-                                  this many peers connected
-        @return: a Deferred that fires with the (string) URI for this file.
+        @return: a Deferred that fires with the UploadResults instance.
+                 To get the URI for this file, use results.uri .
         """
 
-    def create_mutable_file(contents="", wait_for_numpeers=None):
+    def create_mutable_file(contents=""):
         """Create a new mutable file with contents, get back the URI string.
         @param contents: the initial contents to place in the file.
-        @param wait_for_numpeers: don't upload anything until we have at least
-                                  this many peers connected
         @return: a Deferred that fires with tne (string) SSK URI for the new
                  file.
         """
 
-    def create_empty_dirnode(wait_for_numpeers=None):
+    def create_empty_dirnode():
         """Create a new dirnode, empty and unattached.
-        @param wait_for_numpeers: don't create anything until we have at least
-                                  this many peers connected.
         @return: a Deferred that fires with the new IDirectoryNode instance.
         """
 
@@ -1197,6 +1387,59 @@ class IClient(Interface):
                  IDirectoryNode-providing instances, like NewDirectoryNode.
         """
 
+class IClientStatus(Interface):
+    def list_uploads():
+        """Return a list of IUploadStatus objects, one for each
+        upload which is currently running."""
+    def list_downloads():
+        """Return a list of IDownloadStatus objects, one for each
+        download which is currently running."""
+
+class IUploadStatus(Interface):
+    def get_storage_index():
+        """Return a string with the (binary) storage index in use on this
+        upload. Returns None if the storage index has not yet been
+        calculated."""
+    def get_size():
+        """Return an integer with the number of bytes that will eventually
+        be uploaded for this file. Returns None if the size is not yet known.
+        """
+    def using_helper():
+        """Return True if this upload is using a Helper, False if not."""
+    def get_status():
+        """Return a string describing the current state of the upload
+        process."""
+    def get_progress():
+        """Returns a tuple of floats, (chk, ciphertext, encode_and_push),
+        each from 0.0 to 1.0 . 'chk' describes how much progress has been
+        made towards hashing the file to determine a CHK encryption key: if
+        non-convergent encryption is in use, this will be trivial, otherwise
+        the whole file must be hashed. 'ciphertext' describes how much of the
+        ciphertext has been pushed to the helper, and is '1.0' for non-helper
+        uploads. 'encode_and_push' describes how much of the encode-and-push
+        process has finished: for helper uploads this is dependent upon the
+        helper providing progress reports. It might be reasonable to add all
+        three numbers and report the sum to the user."""
+
+class IDownloadStatus(Interface):
+    def get_storage_index():
+        """Return a string with the (binary) storage index in use on this
+        download. This may be None if there is no storage index (i.e. LIT
+        files)."""
+    def get_size():
+        """Return an integer with the number of bytes that will eventually be
+        retrieved for this file. Returns None if the size is not yet known.
+        """
+    def using_helper():
+        """Return True if this download is using a Helper, False if not."""
+    def get_status():
+        """Return a string describing the current state of the download
+        process."""
+    def get_progress():
+        """Returns a float (from 0.0 to 1.0) describing the amount of the
+        download that has completed. This value will remain at 0.0 until the
+        first byte of plaintext is pushed to the download target."""
+
 
 class NotCapableError(Exception):
     """You have tried to write to a read-only node."""
@@ -1233,11 +1476,13 @@ class RIControlClient(RemoteInterface):
         measuring memory consupmtion in bytes."""
         return DictOf(str, int)
 
-    def speed_test(count=int, size=int, mutable=bool):
+    def speed_test(count=int, size=int, mutable=Any()):
         """Write 'count' tempfiles to disk, all of the given size. Measure
         how long (in seconds) it takes to upload them all to the servers.
         Then measure how long it takes to download all of them. If 'mutable'
-        is True, use mutable files instead of immutable ones.
+        is 'create', time creation of mutable files. If 'mutable' is
+        'upload', then time access to the same mutable file instead of
+        creating one.
 
         Returns a tuple of (upload_time, download_time).
         """
@@ -1252,3 +1497,93 @@ class RIControlClient(RemoteInterface):
         """
 
         return DictOf(Nodeid, float)
+
+UploadResults = Any() #DictOf(str, str)
+
+class RIEncryptedUploadable(RemoteInterface):
+    __remote_name__ = "RIEncryptedUploadable.tahoe.allmydata.com"
+
+    def get_size():
+        return int
+
+    def get_all_encoding_parameters():
+        return (int, int, int, long)
+
+    def read_encrypted(offset=long, length=long):
+        return ListOf(str)
+
+    def get_plaintext_hashtree_leaves(first=int, last=int, num_segments=int):
+        return ListOf(Hash)
+
+    def get_plaintext_hash():
+        return Hash
+
+    def close():
+        return None
+
+
+class RICHKUploadHelper(RemoteInterface):
+    __remote_name__ = "RIUploadHelper.tahoe.allmydata.com"
+
+    def upload(reader=RIEncryptedUploadable):
+        return UploadResults
+
+
+class RIHelper(RemoteInterface):
+    __remote_name__ = "RIHelper.tahoe.allmydata.com"
+
+    def upload_chk(si=StorageIndex):
+        """See if a file with a given storage index needs uploading. The
+        helper will ask the appropriate storage servers to see if the file
+        has already been uploaded. If so, the helper will return a set of
+        'upload results' that includes whatever hashes are needed to build
+        the read-cap, and perhaps a truncated sharemap.
+
+        If the file has not yet been uploaded (or if it was only partially
+        uploaded), the helper will return an empty upload-results dictionary
+        and also an RICHKUploadHelper object that will take care of the
+        upload process. The client should call upload() on this object and
+        pass it a reference to an RIEncryptedUploadable object that will
+        provide ciphertext. When the upload is finished, the upload() method
+        will finish and return the upload results.
+        """
+        return (UploadResults, ChoiceOf(RICHKUploadHelper, None))
+
+
+class RIStatsProvider(RemoteInterface):
+    __remote_name__ = "RIStatsProvider.tahoe.allmydata.com"
+    """
+    Provides access to statistics and monitoring information.
+    """
+
+    def get_stats():
+        """
+        returns a dictionary containing 'counters' and 'stats', each a dictionary
+        with string counter/stat name keys, and numeric values.  counters are
+        monotonically increasing measures of work done, and stats are instantaneous
+        measures (potentially time averaged internally)
+        """
+        return DictOf(str, DictOf(str, ChoiceOf(float, int, long)))
+
+class RIStatsGatherer(RemoteInterface):
+    __remote_name__ = "RIStatsGatherer.tahoe.allmydata.com"
+    """
+    Provides a monitoring service for centralised collection of stats
+    """
+
+    def provide(provider=RIStatsProvider, nickname=str):
+        """
+        @param provider: a stats collector instance which should be polled
+                         periodically by the gatherer to collect stats.
+        @param nickname: a name useful to identify the provided client
+        """
+        return None
+
+
+class IStatsProducer(Interface):
+    def get_stats():
+        """
+        returns a dictionary, with str keys representing the names of stats
+        to be monitored, and numeric values.
+        """
+
