@@ -6,9 +6,9 @@ from twisted.internet import defer
 from twisted.python import failure
 from foolscap.eventual import eventually
 from allmydata.interfaces import IMutableFileNode, IMutableFileURI
-from allmydata.util import hashutil, mathutil, idlib, log
+from allmydata.util import base32, hashutil, mathutil, idlib, log
 from allmydata.uri import WriteableSSKFileURI
-from allmydata import hashtree, codec
+from allmydata import hashtree, codec, storage
 from allmydata.encode import NotEnoughPeersError
 from pycryptopp.publickey import rsa
 from pycryptopp.cipher.aes import AES
@@ -207,7 +207,7 @@ class Retrieve:
         self._readkey = filenode.get_readkey()
         self._last_failure = None
         self._log_number = None
-        self._log_prefix = prefix = idlib.b2a(self._storage_index)[:6]
+        self._log_prefix = prefix = storage.si_b2a(self._storage_index)[:5]
         num = self._node._client.log("Retrieve(%s): starting" % prefix)
         self._log_number = num
 
@@ -261,6 +261,12 @@ class Retrieve:
         #    remove that share from the sharemap.  and start step#6 again.
 
         initial_query_count = 5
+        # how much data should be read on the first fetch? It would be nice
+        # if we could grab small directories in a single RTT. The way we pack
+        # dirnodes consumes about 112 bytes per child. The way we pack
+        # mutable files puts about 935 bytes of pubkey+sig+hashes, then our
+        # data, then about 1216 bytes of encprivkey. So 2kB ought to get us
+        # about 9 entries, which seems like a good default.
         self._read_size = 2000
 
         # we might not know how many shares we need yet.
@@ -301,16 +307,16 @@ class Retrieve:
 
     def _choose_initial_peers(self, numqueries):
         n = self._node
-        full_peerlist = n._client.get_permuted_peers(self._storage_index,
-                                                     include_myself=True)
+        full_peerlist = n._client.get_permuted_peers("storage",
+                                                     self._storage_index)
+
         # _peerlist is a list of (peerid,conn) tuples for peers that are
         # worth talking too. This starts with the first numqueries in the
         # permuted list. If that's not enough to get us a recoverable
         # version, we expand this to include the first 2*total_shares peerids
         # (assuming we learn what total_shares is from one of the first
         # numqueries peers)
-        self._peerlist = [(p[1],p[2])
-                          for p in islice(full_peerlist, numqueries)]
+        self._peerlist = [p for p in islice(full_peerlist, numqueries)]
         # _peerlist_limit is the query limit we used to build this list. If
         # we later increase this limit, it may be useful to re-scan the
         # permuted list.
@@ -323,33 +329,20 @@ class Retrieve:
         self._queries_outstanding = set()
         self._used_peers = set()
         self._sharemap = DictOfSets() # shnum -> [(peerid, seqnum, R)..]
-        self._peer_storage_servers = {}
         dl = []
-        for (peerid, conn) in peerlist:
+        for (peerid, ss) in peerlist:
             self._queries_outstanding.add(peerid)
-            self._do_query(conn, peerid, self._storage_index, self._read_size,
-                           self._peer_storage_servers)
+            self._do_query(ss, peerid, self._storage_index, self._read_size)
 
         # control flow beyond this point: state machine. Receiving responses
         # from queries is the input. We might send out more queries, or we
         # might produce a result.
         return None
 
-    def _do_query(self, conn, peerid, storage_index, readsize,
-                  peer_storage_servers):
+    def _do_query(self, ss, peerid, storage_index, readsize):
         self._queries_outstanding.add(peerid)
-        if peerid in peer_storage_servers:
-            d = defer.succeed(peer_storage_servers[peerid])
-        else:
-            d = conn.callRemote("get_service", "storageserver")
-            def _got_storageserver(ss):
-                peer_storage_servers[peerid] = ss
-                return ss
-            d.addCallback(_got_storageserver)
-        d.addCallback(lambda ss: ss.callRemote("slot_readv", storage_index,
-                                               [], [(0, readsize)]))
-        d.addCallback(self._got_results, peerid, readsize,
-                      (conn, storage_index, peer_storage_servers))
+        d = ss.callRemote("slot_readv", storage_index, [], [(0, readsize)])
+        d.addCallback(self._got_results, peerid, readsize, (ss, storage_index))
         d.addErrback(self._query_failed, peerid)
         # errors that aren't handled by _query_failed (and errors caused by
         # _query_failed) get logged, but we still want to check for doneness.
@@ -377,9 +370,8 @@ class Retrieve:
                 # TODO: for MDMF, sanity-check self._read_size: don't let one
                 # server cause us to try to read gigabytes of data from all
                 # other servers.
-                (conn, storage_index, peer_storage_servers) = stuff
-                self._do_query(conn, peerid, storage_index, self._read_size,
-                               peer_storage_servers)
+                (ss, storage_index) = stuff
+                self._do_query(ss, peerid, storage_index, self._read_size)
                 return
             except CorruptShareError, e:
                 # log it and give the other shares a chance to be processed
@@ -418,12 +410,15 @@ class Retrieve:
             # ok, it's a valid verinfo. Add it to the list of validated
             # versions.
             self.log(" found valid version %d-%s from %s-sh%d: %d-%d/%d/%d"
-                     % (seqnum, idlib.b2a(root_hash)[:4],
+                     % (seqnum, base32.b2a(root_hash)[:4],
                         idlib.shortnodeid_b2a(peerid), shnum,
                         k, N, segsize, datalength))
             self._valid_versions[verinfo] = (prefix, DictOfSets())
 
             # and make a note of the other parameters we've just learned
+            # NOTE: Retrieve needs to be refactored to put k,N in the verinfo
+            # along with seqnum/etc, to make sure we don't co-mingle shares
+            # from differently-encoded versions of the same file.
             if self._required_shares is None:
                 self._required_shares = k
                 self._node._populate_required_shares(k)
@@ -431,6 +426,23 @@ class Retrieve:
                 self._total_shares = N
                 self._node._populate_total_shares(N)
 
+        # reject shares that don't match our narrow-minded ideas of what
+        # encoding we're going to use. This addresses the immediate needs of
+        # ticket #312, by turning the data corruption into unavailability. To
+        # get back the availability (i.e. make sure that one weird-encoding
+        # share that happens to come back first doesn't make us ignore the
+        # rest of the shares), we need to implement the refactoring mentioned
+        # above.
+        if k != self._required_shares:
+            raise CorruptShareError(peerid, shnum,
+                                    "share has k=%d, we want k=%d" %
+                                    (k, self._required_shares))
+
+        if N != self._total_shares:
+            raise CorruptShareError(peerid, shnum,
+                                    "share has N=%d, we want N=%d" %
+                                    (N, self._total_shares))
+
         # we've already seen this pair, and checked the signature so we
         # know it's a valid candidate. Accumulate the share info, if
         # there's enough data present. If not, raise NeedMoreDataError,
@@ -514,19 +526,18 @@ class Retrieve:
         self.log("search_distance=%d" % search_distance, level=log.UNUSUAL)
         if self._peerlist_limit < search_distance:
             # we might be able to get some more peers from the list
-            peers = self._node._client.get_permuted_peers(self._storage_index,
-                                                          include_myself=True)
-            self._peerlist = [(p[1],p[2])
-                              for p in islice(peers, search_distance)]
+            peers = self._node._client.get_permuted_peers("storage",
+                                                          self._storage_index)
+            self._peerlist = [p for p in islice(peers, search_distance)]
             self._peerlist_limit = search_distance
             self.log("added peers, peerlist=%d, peerlist_limit=%d"
                      % (len(self._peerlist), self._peerlist_limit),
                      level=log.UNUSUAL)
         # are there any peers on the list that we haven't used?
         new_query_peers = []
-        for (peerid, conn) in self._peerlist:
+        for (peerid, ss) in self._peerlist:
             if peerid not in self._used_peers:
-                new_query_peers.append( (peerid, conn) )
+                new_query_peers.append( (peerid, ss) )
                 if len(new_query_peers) > 5:
                     # only query in batches of 5. TODO: this is pretty
                     # arbitrary, really I want this to be something like
@@ -535,10 +546,8 @@ class Retrieve:
         if new_query_peers:
             self.log("sending %d new queries (read %d bytes)" %
                      (len(new_query_peers), self._read_size), level=log.UNUSUAL)
-            for (peerid, conn) in new_query_peers:
-                self._do_query(conn, peerid,
-                               self._storage_index, self._read_size,
-                               self._peer_storage_servers)
+            for (peerid, ss) in new_query_peers:
+                self._do_query(ss, peerid, self._storage_index, self._read_size)
             # we'll retrigger when those queries come back
             return
 
@@ -559,7 +568,7 @@ class Retrieve:
                 shares_s.append("#%d" % shnum)
         shares_s = ",".join(shares_s)
         self.log("_attempt_decode: version %d-%s, shares: %s" %
-                 (seqnum, idlib.b2a(root_hash)[:4], shares_s))
+                 (seqnum, base32.b2a(root_hash)[:4], shares_s))
 
         # first, validate each share that we haven't validated yet. We use
         # self._valid_shares to remember which ones we've already checked.
@@ -712,44 +721,29 @@ class Publish:
     def __init__(self, filenode):
         self._node = filenode
         self._storage_index = self._node.get_storage_index()
-        self._log_prefix = prefix = idlib.b2a(self._storage_index)[:6]
+        self._log_prefix = prefix = storage.si_b2a(self._storage_index)[:5]
         num = self._node._client.log("Publish(%s): starting" % prefix)
         self._log_number = num
 
-    def log(self, msg, parent=None, **kwargs):
-        prefix = self._log_prefix
-        if parent is None:
-            parent = self._log_number
-        num = self._node._client.log("Publish(%s): %s" % (prefix, msg),
-                                     parent=parent, **kwargs)
+    def log(self, *args, **kwargs):
+        if 'parent' not in kwargs:
+            kwargs['parent'] = self._log_number
+        num = log.msg(*args, **kwargs)
         return num
 
-    def log_err(self, f):
-        num = log.err(f, parent=self._log_number)
+    def log_err(self, *args, **kwargs):
+        if 'parent' not in kwargs:
+            kwargs['parent'] = self._log_number
+        num = log.err(*args, **kwargs)
         return num
 
-    def publish(self, newdata, wait_for_numpeers=None):
+    def publish(self, newdata):
         """Publish the filenode's current contents.  Returns a Deferred that
         fires (with None) when the publish has done as much work as it's ever
         going to do, or errbacks with ConsistencyError if it detects a
         simultaneous write.
-
-        It will wait until at least wait_for_numpeers peers are connected
-        before it starts uploading
-
-        If wait_for_numpeers is None then it will be set to a default value
-        (currently 1).
         """
-        if wait_for_numpeers is None:
-            wait_for_numpeers = 1
 
-        self.log("starting publish")
-
-        d = self._node._client.introducer_client.when_enough_peers(wait_for_numpeers)
-        d.addCallback(lambda dummy: self._after_enough_peers(newdata))
-        return d
-
-    def _after_enough_peers(self, newdata):
         # 1: generate shares (SDMF: files are small, so we can do it in RAM)
         # 2: perform peer selection, get candidate servers
         #  2a: send queries to n+epsilon servers, to determine current shares
@@ -759,7 +753,7 @@ class Publish:
         # 4a: may need to run recovery algorithm
         # 5: when enough responses are back, we're done
 
-        self.log("got enough peers, datalen is %s" % len(newdata))
+        self.log("starting publish, datalen is %s" % len(newdata))
 
         self._writekey = self._node.get_writekey()
         assert self._writekey, "need write capability to publish"
@@ -786,8 +780,10 @@ class Publish:
         # the privkey from somebody, which means reading more like 3KB, but
         # the code in _obtain_privkey will ensure that we manage that even if
         # we need an extra roundtrip. TODO: arrange to read 3KB from one peer
-        # who is likely to hold a share (like, say, ourselves), so we can
-        # avoid the latency of that extra roundtrip.
+        # who is likely to hold a share, so we can avoid the latency of that
+        # extra roundtrip. 3KB would get us the encprivkey from a dirnode
+        # with up to 7 entries, allowing us to make an update in 2 RTT
+        # instead of 3.
         self._read_size = 1000
 
         d = defer.succeed(total_shares)
@@ -808,36 +804,39 @@ class Publish:
 
         storage_index = self._storage_index
 
-        # we need to include ourselves in the list for two reasons. The most
-        # important is so that any shares which already exist on our own
-        # server get updated. The second is to ensure that we leave a share
-        # on our own server, so we're more likely to have the signing key
-        # around later. This way, even if all the servers die and the
-        # directory contents are unrecoverable, at least we can still push
-        # out a new copy with brand-new contents. TODO: it would be nice if
-        # the share we use for ourselves didn't count against the N total..
-        # maybe use N+1 if we find ourselves in the permuted list?
+        # In 0.7.0, we went through extra work to make sure that we include
+        # ourselves in the peerlist, mainly to match Retrieve (which did the
+        # same thing. With the post-0.7.0 Introducer refactoring, we got rid
+        # of the include-myself flags, and standardized on the
+        # uploading/downloading node not being special.
 
-        peerlist = self._node._client.get_permuted_peers(storage_index,
-                                                         include_myself=True)
+        # One nice feature of the old approach was that by putting a share on
+        # the local storage server, we're more likely to be able to retrieve
+        # a copy of the encrypted private key (even if all the old servers
+        # have gone away), so we can regenerate new shares even if we can't
+        # retrieve the old contents. This need will eventually go away when
+        # we switch to DSA-based mutable files (which store the private key
+        # in the URI).
+
+        peerlist = self._node._client.get_permuted_peers("storage",
+                                                         storage_index)
 
         current_share_peers = DictOfSets()
         reachable_peers = {}
-        # list of (peerid, offset, length) where the encprivkey might be found
+        # list of (peerid, shnum, offset, length) where the encprivkey might
+        # be found
         self._encprivkey_shares = []
 
         EPSILON = total_shares / 2
         #partial_peerlist = islice(peerlist, total_shares + EPSILON)
         partial_peerlist = peerlist[:total_shares+EPSILON]
 
-        # make sure our local server is in the list
-        partial_peerlist = self._add_ourselves(partial_peerlist, peerlist)
+        self._storage_servers = {}
 
-        peer_storage_servers = {}
         dl = []
-        for (permutedid, peerid, conn) in partial_peerlist:
-            d = self._do_query(conn, peerid, peer_storage_servers,
-                               storage_index)
+        for permutedid, (peerid, ss) in enumerate(partial_peerlist):
+            self._storage_servers[peerid] = ss
+            d = self._do_query(ss, peerid, storage_index)
             d.addCallback(self._got_query_results,
                           peerid, permutedid,
                           reachable_peers, current_share_peers)
@@ -845,50 +844,34 @@ class Publish:
         d = defer.DeferredList(dl)
         d.addCallback(self._got_all_query_results,
                       total_shares, reachable_peers,
-                      current_share_peers, peer_storage_servers)
+                      current_share_peers)
         # TODO: add an errback to, probably to ignore that peer
+
         # TODO: if we can't get a privkey from these servers, consider
-        # looking farther afield. Make sure we include ourselves in the
-        # initial list, because of the 0.7.0 behavior that causes us to
-        # create our initial directory before we've connected to anyone
-        # but ourselves.
+        # looking farther afield. Be aware of the old 0.7.0 behavior that
+        # causes us to create our initial directory before we've connected to
+        # anyone but ourselves.. those old directories may not be
+        # retrieveable if our own server is no longer in the early part of
+        # the permuted peerlist.
         return d
 
-    def _add_ourselves(self, partial_peerlist, peerlist):
-        my_peerid = self._node._client.nodeid
-        for (permutedid, peerid, conn) in partial_peerlist:
-            if peerid == my_peerid:
-                # we're already in there
-                return partial_peerlist
-        for (permutedid, peerid, conn) in peerlist:
-            if peerid == self._node._client.nodeid:
-                # found it
-                partial_peerlist.append( (permutedid, peerid, conn) )
-                return partial_peerlist
-        self.log("we aren't in our own peerlist??", level=log.WEIRD)
-        return partial_peerlist
-
-    def _do_query(self, conn, peerid, peer_storage_servers, storage_index):
+    def _do_query(self, ss, peerid, storage_index):
         self.log("querying %s" % idlib.shortnodeid_b2a(peerid))
-        d = conn.callRemote("get_service", "storageserver")
-        def _got_storageserver(ss):
-            peer_storage_servers[peerid] = ss
-            return ss.callRemote("slot_readv",
-                                 storage_index, [], [(0, self._read_size)])
-        d.addCallback(_got_storageserver)
+        d = ss.callRemote("slot_readv",
+                          storage_index, [], [(0, self._read_size)])
         return d
 
     def _got_query_results(self, datavs, peerid, permutedid,
                            reachable_peers, current_share_peers):
 
-        lp = self.log("_got_query_results from %s" %
-                      idlib.shortnodeid_b2a(peerid))
+        lp = self.log(format="_got_query_results from %(peerid)s",
+                      peerid=idlib.shortnodeid_b2a(peerid))
         assert isinstance(datavs, dict)
         reachable_peers[peerid] = permutedid
         if not datavs:
             self.log("peer has no shares", parent=lp)
         for shnum, datav in datavs.items():
-            self.log("peer has shnum %d" % shnum, parent=lp)
+            lp2 = self.log("peer has shnum %d" % shnum, parent=lp)
             assert len(datav) == 1
             data = datav[0]
             # We want (seqnum, root_hash, IV) from all servers to know what
@@ -907,10 +890,13 @@ class Publish:
             # self._pubkey is present because we require read-before-replace
             valid = self._pubkey.verify(prefix, signature)
             if not valid:
-                self.log("bad signature from %s shnum %d" %
-                         (shnum, idlib.shortnodeid_b2a(peerid)),
-                         parent=lp, level=log.WEIRD)
+                self.log(format="bad signature from %(peerid)s shnum %(shnum)d",
+                         peerid=idlib.shortnodeid_b2a(peerid), shnum=shnum,
+                         parent=lp2, level=log.WEIRD)
                 continue
+            self.log(format="peer has goodsig shnum %(shnum)d seqnum %(seqnum)d",
+                     shnum=shnum, seqnum=seqnum,
+                     parent=lp2, level=log.NOISY)
 
             share = (shnum, seqnum, root_hash)
             current_share_peers.add(shnum, (peerid, seqnum, root_hash) )
@@ -939,7 +925,7 @@ class Publish:
             # files (since the privkey will be small enough to fit in the
             # write cap).
 
-            self._encprivkey_shares.append( (peerid, shnum, offset, length) )
+            self._encprivkey_shares.append( (peerid, shnum, offset, length))
             return
 
         (seqnum, root_hash, IV, k, N, segsize, datalen,
@@ -966,7 +952,7 @@ class Publish:
 
     def _got_all_query_results(self, res,
                                total_shares, reachable_peers,
-                               current_share_peers, peer_storage_servers):
+                               current_share_peers):
         self.log("_got_all_query_results")
         # now that we know everything about the shares currently out there,
         # decide where to place the new shares.
@@ -978,6 +964,19 @@ class Publish:
         # TODO: 2: move those shares instead of copying them, to reduce future
         #       update work
 
+        # if log.recording_noisy
+        logmsg = []
+        for shnum in range(total_shares):
+            logmsg2 = []
+            for oldplace in current_share_peers.get(shnum, []):
+                (peerid, seqnum, R) = oldplace
+                logmsg2.append("%s:#%d:R=%s" % (idlib.shortnodeid_b2a(peerid),
+                                                seqnum, base32.b2a(R)[:4]))
+            logmsg.append("sh%d on (%s)" % (shnum, "/".join(logmsg2)))
+        self.log("sharemap: %s" % (", ".join(logmsg)), level=log.NOISY)
+        self.log("we are planning to push new seqnum=#%d" % self._new_seqnum,
+                 level=log.NOISY)
+
         shares_needing_homes = range(total_shares)
         target_map = DictOfSets() # maps shnum to set((peerid,oldseqnum,oldR))
         shares_per_peer = DictOfSets()
@@ -985,7 +984,13 @@ class Publish:
             for oldplace in current_share_peers.get(shnum, []):
                 (peerid, seqnum, R) = oldplace
                 if seqnum >= self._new_seqnum:
-                    raise UncoordinatedWriteError()
+                    self.log("somebody has a newer sequence number than what we were uploading",
+                             level=log.WEIRD)
+                    self.log(format="peerid=%(peerid)s, theirs=%(seqnum)d, mine=%(new_seqnum)d",
+                             peerid=idlib.shortnodeid_b2a(peerid),
+                             seqnum=seqnum,
+                             new_seqnum=self._new_seqnum)
+                    raise UncoordinatedWriteError("somebody has a newer sequence number than us")
                 target_map.add(shnum, oldplace)
                 shares_per_peer.add(peerid, shnum)
                 if shnum in shares_needing_homes:
@@ -997,7 +1002,7 @@ class Publish:
         # one-per-peer in the normal permuted order.
         while shares_needing_homes:
             if not reachable_peers:
-                prefix = idlib.b2a(self._node.get_storage_index())[:6]
+                prefix = storage.si_b2a(self._node.get_storage_index())[:5]
                 raise NotEnoughPeersError("ran out of peers during upload of (%s); shares_needing_homes: %s, reachable_peers: %s" % (prefix, shares_needing_homes, reachable_peers,))
             shnum = shares_needing_homes.pop(0)
             possible_homes = reachable_peers.keys()
@@ -1012,7 +1017,7 @@ class Publish:
 
         assert not shares_needing_homes
 
-        target_info = (target_map, shares_per_peer, peer_storage_servers)
+        target_info = (target_map, shares_per_peer)
         return target_info
 
     def _obtain_privkey(self, target_info):
@@ -1025,16 +1030,16 @@ class Publish:
         # peers one at a time until we get a copy. Only bother asking peers
         # who've admitted to holding a share.
 
-        target_map, shares_per_peer, peer_storage_servers = target_info
+        target_map, shares_per_peer = target_info
         # pull shares from self._encprivkey_shares
         if not self._encprivkey_shares:
             raise NotEnoughPeersError("Unable to find a copy of the privkey")
 
         (peerid, shnum, offset, length) = self._encprivkey_shares.pop(0)
+        ss = self._storage_servers[peerid]
         self.log("trying to obtain privkey from %s shnum %d" %
                  (idlib.shortnodeid_b2a(peerid), shnum))
-        d = self._do_privkey_query(peer_storage_servers[peerid], peerid,
-                                   shnum, offset, length)
+        d = self._do_privkey_query(ss, peerid, shnum, offset, length)
         d.addErrback(self.log_err)
         d.addCallback(lambda res: self._obtain_privkey(target_info))
         return d
@@ -1129,7 +1134,7 @@ class Publish:
                                               for i in needed_hashes ] )
         root_hash = share_hash_tree[0]
         assert len(root_hash) == 32
-        self.log("my new root_hash is %s" % idlib.b2a(root_hash))
+        self.log("my new root_hash is %s" % base32.b2a(root_hash))
 
         prefix = pack_prefix(seqnum, root_hash, IV,
                              required_shares, total_shares,
@@ -1167,7 +1172,7 @@ class Publish:
         # surprises here are *not* indications of UncoordinatedWriteError,
         # and we'll need to respond to them more gracefully.)
 
-        target_map, shares_per_peer, peer_storage_servers = target_info
+        target_map, shares_per_peer = target_info
 
         my_checkstring = pack_checkstring(seqnum, root_hash, IV)
         peer_messages = {}
@@ -1199,7 +1204,7 @@ class Publish:
             cancel_secret = self._node.get_cancel_secret(peerid)
             secrets = (write_enabler, renew_secret, cancel_secret)
 
-            d = self._do_testreadwrite(peerid, peer_storage_servers, secrets,
+            d = self._do_testreadwrite(peerid, secrets,
                                        tw_vectors, read_vector)
             d.addCallback(self._got_write_answer, tw_vectors, my_checkstring,
                           peerid, expected_old_shares[peerid], dispatch_map)
@@ -1209,22 +1214,23 @@ class Publish:
         d.addCallback(lambda res: (self._surprised, dispatch_map))
         return d
 
-    def _do_testreadwrite(self, peerid, peer_storage_servers, secrets,
+    def _do_testreadwrite(self, peerid, secrets,
                           tw_vectors, read_vector):
-        conn = peer_storage_servers[peerid]
         storage_index = self._node._uri.storage_index
+        ss = self._storage_servers[peerid]
 
-        d = conn.callRemote("slot_testv_and_readv_and_writev",
-                            storage_index,
-                            secrets,
-                            tw_vectors,
-                            read_vector)
+        d = ss.callRemote("slot_testv_and_readv_and_writev",
+                          storage_index,
+                          secrets,
+                          tw_vectors,
+                          read_vector)
         return d
 
     def _got_write_answer(self, answer, tw_vectors, my_checkstring,
                           peerid, expected_old_shares,
                           dispatch_map):
-        self.log("_got_write_answer from %s" % idlib.shortnodeid_b2a(peerid))
+        lp = self.log("_got_write_answer from %s" %
+                      idlib.shortnodeid_b2a(peerid))
         wrote, read_data = answer
         surprised = False
 
@@ -1235,6 +1241,8 @@ class Publish:
                 dispatch_map.add(shnum, (peerid, new_seqnum, new_root_hash))
         else:
             # surprise! our testv failed, so the write did not happen
+            self.log("our testv failed, that write did not happen",
+                     parent=lp, level=log.WEIRD)
             surprised = True
 
         for shnum, (old_cs,) in read_data.items():
@@ -1245,12 +1253,21 @@ class Publish:
 
             if shnum not in expected_old_shares:
                 # surprise! there was a share we didn't know about
+                self.log("they had share %d that we didn't know about" % shnum,
+                         parent=lp, level=log.WEIRD)
                 surprised = True
             else:
                 seqnum, root_hash = expected_old_shares[shnum]
                 if seqnum is not None:
                     if seqnum != old_seqnum or root_hash != old_root_hash:
                         # surprise! somebody modified the share on us
+                        self.log("somebody modified the share on us:"
+                                 " shnum=%d: I thought they had #%d:R=%s,"
+                                 " but testv reported #%d:R=%s" %
+                                 (shnum,
+                                  seqnum, base32.b2a(root_hash)[:4],
+                                  old_seqnum, base32.b2a(old_root_hash)[:4]),
+                                 parent=lp, level=log.WEIRD)
                         surprised = True
         if surprised:
             self._surprised = True
@@ -1259,16 +1276,19 @@ class Publish:
         for shnum, places in dispatch_map.items():
             sent_to = [(idlib.shortnodeid_b2a(peerid),
                         seqnum,
-                        idlib.b2a(root_hash)[:4])
+                        base32.b2a(root_hash)[:4])
                        for (peerid,seqnum,root_hash) in places]
-            self.log(" share %d sent to: %s" % (shnum, sent_to))
+            self.log(" share %d sent to: %s" % (shnum, sent_to),
+                     level=log.NOISY)
 
     def _maybe_recover(self, (surprised, dispatch_map)):
-        self.log("_maybe_recover, surprised=%s, dispatch_map:" % surprised)
+        self.log("_maybe_recover, surprised=%s, dispatch_map:" % surprised,
+                 level=log.NOISY)
         self._log_dispatch_map(dispatch_map)
         if not surprised:
             self.log(" no recovery needed")
             return
+        self.log("We need recovery!", level=log.WEIRD)
         print "RECOVERY NOT YET IMPLEMENTED"
         # but dispatch_map will help us do it
         raise UncoordinatedWriteError("I was surprised!")
@@ -1318,7 +1338,7 @@ class MutableFileNode:
         self._encprivkey = None
         return self
 
-    def create(self, initial_contents, wait_for_numpeers=None):
+    def create(self, initial_contents):
         """Call this when the filenode is first created. This will generate
         the keys, generate the initial shares, wait until at least numpeers
         are connected, allocate shares, and upload the initial
@@ -1342,7 +1362,7 @@ class MutableFileNode:
             # nobody knows about us yet"
             self._current_seqnum = 0
             self._current_roothash = "\x00"*32
-            return self._publish(initial_contents, wait_for_numpeers=wait_for_numpeers)
+            return self._publish(initial_contents)
         d.addCallback(_generated)
         return d
 
@@ -1352,9 +1372,9 @@ class MutableFileNode:
         verifier = signer.get_verifying_key()
         return verifier, signer
 
-    def _publish(self, initial_contents, wait_for_numpeers):
+    def _publish(self, initial_contents):
         p = self.publish_class(self)
-        d = p.publish(initial_contents, wait_for_numpeers=wait_for_numpeers)
+        d = p.publish(initial_contents)
         d.addCallback(lambda res: self)
         return d
 
@@ -1474,8 +1494,8 @@ class MutableFileNode:
         r = Retrieve(self)
         return r.retrieve()
 
-    def replace(self, newdata, wait_for_numpeers=None):
+    def replace(self, newdata):
         r = Retrieve(self)
         d = r.retrieve()
-        d.addCallback(lambda res: self._publish(newdata, wait_for_numpeers=wait_for_numpeers))
+        d.addCallback(lambda res: self._publish(newdata))
         return d
