@@ -161,7 +161,6 @@ class Node(service.MultiService):
         d = defer.succeed(None)
         d.addCallback(lambda res: iputil.get_local_addresses_async())
         d.addCallback(self._setup_tub)
-        d.addCallback(lambda res: self.tub_ready())
         def _ready(res):
             self.log("%s running" % self.NODETYPE)
             self._tub_ready_observerlist.fire(self)
@@ -170,8 +169,11 @@ class Node(service.MultiService):
         def _die(failure):
             self.log('_startService() failed')
             log.err(failure)
+            print "Node._startService failed, aborting"
+            print failure
             #reactor.stop() # for unknown reasons, reactor.stop() isn't working.  [ ] TODO
             self.log('calling os.abort()')
+            print "calling os.abort()"
             os.abort()
         d.addErrback(_die)
 
@@ -208,7 +210,10 @@ class Node(service.MultiService):
                            os.path.join(self.basedir, "log_gatherer.furl"))
         self.tub.setOption("bridge-twisted-logs", True)
 
-    def log(self, msg, src="", args=(), **kw):
+    def log(self, *args, **kwargs):
+        return tahoe_log.msg(*args, **kwargs)
+
+    def old_log(self, msg, src="", args=(), **kw):
         if src:
             logsrc = src
         else:
@@ -251,10 +256,6 @@ class Node(service.MultiService):
         self.tub.setLocation(location)
         return self.tub
 
-    def tub_ready(self):
-        # called when the Tub is available for registerReference
-        pass
-
     def when_tub_ready(self):
         return self._tub_ready_observerlist.when_fired()
 
