@@ -1,11 +1,10 @@
 
 from nevow import inevow, loaders, rend, tags as T
-from twisted.python import util
 import math
-from allmydata.util import mathutil
+from allmydata.util import mathutil, sibpath
 
 def getxmlfile(name):
-    return loaders.xmlfile(util.sibpath(__file__, "web/%s" % name))
+    return loaders.xmlfile(sibpath.sibpath(__file__, "web/%s" % name))
 
 # factorial and binomial copied from
 # http://mail.python.org/pipermail/python-list/2007-April/435718.html
