@@ -8,19 +8,24 @@ from twisted.internet import defer
 from zope.interface import implements
 from allmydata.interfaces import RIStorageServer, RIBucketWriter, \
      RIBucketReader, IStorageBucketWriter, IStorageBucketReader, HASH_SIZE, \
-     BadWriteEnablerError
-from allmydata.util import fileutil, idlib, mathutil
+     BadWriteEnablerError, IStatsProducer
+from allmydata.util import base32, fileutil, idlib, mathutil, log
 from allmydata.util.assertutil import precondition, _assert
+import allmydata # for __version__
 
 class DataTooLargeError(Exception):
     pass
 
 # storage/
 # storage/shares/incoming
-#   incoming/ holds temp dirs named $STORAGEINDEX/$SHARENUM which will be
-#   moved to storage/shares/$STORAGEINDEX/$SHARENUM upon success
-# storage/shares/$STORAGEINDEX
-# storage/shares/$STORAGEINDEX/$SHARENUM
+#   incoming/ holds temp dirs named $START/$STORAGEINDEX/$SHARENUM which will
+#   be moved to storage/shares/$START/$STORAGEINDEX/$SHARENUM upon success
+# storage/shares/$START/$STORAGEINDEX
+# storage/shares/$START/$STORAGEINDEX/$SHARENUM
+
+# Where "$START" denotes the first 14 bits worth of $STORAGEINDEX (that's 3
+# base-32 chars, but the last one has only 4 bits in it -- i.e. only 16 possible
+# chars in the last position).
 
 # $SHARENUM matches this regex:
 NUM_RE=re.compile("^[0-9]+$")
@@ -42,6 +47,16 @@ NUM_RE=re.compile("^[0-9]+$")
 #   B+0x44: expiration time, 4 bytes big-endian seconds-since-epoch
 #   B+0x48: next lease, or end of record
 
+def si_b2a(storageindex):
+    return base32.b2a(storageindex)
+
+def si_a2b(ascii_storageindex):
+    return base32.a2b(ascii_storageindex)
+
+def storage_index_to_dir(storageindex):
+    sia = si_b2a(storageindex)
+    return os.path.join(sia[:2], sia)
+
 class ShareFile:
     LEASE_SIZE = struct.calcsize(">L32s32sL")
 
@@ -164,20 +179,23 @@ class ShareFile:
 class BucketWriter(Referenceable):
     implements(RIBucketWriter)
 
-    def __init__(self, ss, incominghome, finalhome, size, lease_info):
+    def __init__(self, ss, incominghome, finalhome, size, lease_info, canary):
         self.ss = ss
         self.incominghome = incominghome
         self.finalhome = finalhome
         self._size = size
+        self._canary = canary
+        self._disconnect_marker = canary.notifyOnDisconnect(self._disconnected)
         self.closed = False
         self.throw_out_all_data = False
         # touch the file, so later callers will see that we're working on it.
+        assert not os.path.exists(incominghome)
+        fileutil.make_dirs(os.path.dirname(incominghome))
         # Also construct the metadata.
-        assert not os.path.exists(self.incominghome)
-        f = open(self.incominghome, 'wb')
+        f = open(incominghome, 'wb')
         f.write(struct.pack(">LLL", 1, size, 0))
         f.close()
-        self._sharefile = ShareFile(self.incominghome)
+        self._sharefile = ShareFile(incominghome)
         # also, add our lease to the file now, so that other ones can be
         # added by simultaneous uploaders
         self._sharefile.add_lease(lease_info)
@@ -193,13 +211,37 @@ class BucketWriter(Referenceable):
 
     def remote_close(self):
         precondition(not self.closed)
+
+        fileutil.make_dirs(os.path.dirname(self.finalhome))
         fileutil.rename(self.incominghome, self.finalhome)
+        try:
+            os.rmdir(os.path.dirname(self.incominghome))
+            os.rmdir(os.path.dirname(os.path.dirname(self.incominghome)))
+            os.rmdir(os.path.dirname(os.path.dirname(os.path.dirname(self.incominghome))))
+        except EnvironmentError:
+            pass
         self._sharefile = None
         self.closed = True
+        self._canary.dontNotifyOnDisconnect(self._disconnect_marker)
 
         filelen = os.stat(self.finalhome)[stat.ST_SIZE]
         self.ss.bucket_writer_closed(self, filelen)
 
+    def _disconnected(self):
+        if not self.closed:
+            self._abort()
+
+    def remote_abort(self):
+        log.msg("storage: aborting sharefile %s" % self.incominghome,
+                facility="tahoe.storage", level=log.UNUSUAL)
+        if not self.closed:
+            self._canary.dontNotifyOnDisconnect(self._disconnect_marker)
+        self._abort()
+
+    def _abort(self):
+        if self.closed:
+            return
+        os.remove(self.incominghome)
         # if we were the last share to be moved, remove the incoming/
         # directory that was our parent
         parentdir = os.path.split(self.incominghome)[0]
@@ -207,11 +249,12 @@ class BucketWriter(Referenceable):
             os.rmdir(parentdir)
 
 
+
 class BucketReader(Referenceable):
     implements(RIBucketReader)
 
-    def __init__(self, home):
-        self._share_file = ShareFile(home)
+    def __init__(self, sharefname):
+        self._share_file = ShareFile(sharefname)
 
     def remote_read(self, offset, length):
         return self._share_file.read_share_data(offset, length)
@@ -257,7 +300,7 @@ class MutableShareFile:
     MAX_SIZE = 2*1000*1000*1000 # 2GB, kind of arbitrary
     # TODO: decide upon a policy for max share size
 
-    def __init__(self, filename):
+    def __init__(self, filename, parent=None):
         self.home = filename
         if os.path.exists(self.home):
             # we don't cache anything, just check the magic
@@ -268,7 +311,10 @@ class MutableShareFile:
              data_length, extra_least_offset) = \
              struct.unpack(">32s20s32sQQ", data)
             assert magic == self.MAGIC
+        self.parent = parent # for logging
 
+    def log(self, *args, **kwargs):
+        return self.parent.log(*args, **kwargs)
 
     def create(self, my_nodeid, write_enabler):
         assert not os.path.exists(self.home)
@@ -555,7 +601,7 @@ class MutableShareFile:
 #        f.close()
 #        return data_length
 
-    def check_write_enabler(self, write_enabler):
+    def check_write_enabler(self, write_enabler, si_s):
         f = open(self.home, 'rb+')
         (real_write_enabler, write_enabler_nodeid) = \
                              self._read_write_enabler_and_nodeid(f)
@@ -563,6 +609,11 @@ class MutableShareFile:
         if write_enabler != real_write_enabler:
             # accomodate share migration by reporting the nodeid used for the
             # old write enabler.
+            self.log(format="bad write enabler on SI %(si)s,"
+                     " recorded by nodeid %(nodeid)s",
+                     facility="tahoe.storage",
+                     level=log.WEIRD,
+                     si=si_s, nodeid=idlib.nodeid_b2a(write_enabler_nodeid))
             msg = "The write enabler was recorded by nodeid '%s'." % \
                   (idlib.nodeid_b2a(write_enabler_nodeid),)
             raise BadWriteEnablerError(msg)
@@ -615,34 +666,49 @@ class EmptyShare:
                 break
         return test_good
 
-def create_mutable_sharefile(filename, my_nodeid, write_enabler):
-    ms = MutableShareFile(filename)
+def create_mutable_sharefile(filename, my_nodeid, write_enabler, parent):
+    ms = MutableShareFile(filename, parent)
     ms.create(my_nodeid, write_enabler)
     del ms
-    return MutableShareFile(filename)
+    return MutableShareFile(filename, parent)
 
 
 class StorageServer(service.MultiService, Referenceable):
-    implements(RIStorageServer)
-    name = 'storageserver'
+    implements(RIStorageServer, IStatsProducer)
+    name = 'storage'
 
-    def __init__(self, storedir, sizelimit=None, no_storage=False):
+    # we're pretty narrow-minded right now
+    OLDEST_SUPPORTED_VERSION = allmydata.__version__
+
+    def __init__(self, storedir, sizelimit=None,
+                 discard_storage=False, readonly_storage=False,
+                 stats_provider=None):
         service.MultiService.__init__(self)
         self.storedir = storedir
         sharedir = os.path.join(storedir, "shares")
         fileutil.make_dirs(sharedir)
         self.sharedir = sharedir
         self.sizelimit = sizelimit
-        self.no_storage = no_storage
+        self.no_storage = discard_storage
+        self.readonly_storage = readonly_storage
+        self.stats_provider = stats_provider
+        if self.stats_provider:
+            self.stats_provider.register_producer(self)
         self.incomingdir = os.path.join(sharedir, 'incoming')
         self._clean_incomplete()
         fileutil.make_dirs(self.incomingdir)
         self._active_writers = weakref.WeakKeyDictionary()
+        lp = log.msg("StorageServer created, now measuring space..",
+                     facility="tahoe.storage")
         self.measure_size()
+        log.msg(format="space measurement done, consumed=%(consumed)d bytes",
+                consumed=self.consumed,
+                parent=lp, facility="tahoe.storage")
 
-    def log(self, msg):
-        #self.parent.log(msg)
-        return
+    def log(self, *args, **kwargs):
+        if "facility" not in kwargs:
+            kwargs["facility"] = "tahoe.storage"
+        return log.msg(*args, **kwargs)
 
     def setNodeID(self, nodeid):
         # somebody must set this before any slots can be created or leases
@@ -659,6 +725,11 @@ class StorageServer(service.MultiService, Referenceable):
     def _clean_incomplete(self):
         fileutil.rm_dir(self.incomingdir)
 
+    def get_stats(self):
+        return { 'storage_server.consumed': self.consumed,
+                 'storage_server.allocated': self.allocated_size(),
+               }
+
     def measure_size(self):
         self.consumed = fileutil.du(self.sharedir)
 
@@ -668,6 +739,9 @@ class StorageServer(service.MultiService, Referenceable):
             space += bw.allocated_size()
         return space
 
+    def remote_get_versions(self):
+        return (str(allmydata.__version__), str(self.OLDEST_SUPPORTED_VERSION))
+
     def remote_allocate_buckets(self, storage_index,
                                 renew_secret, cancel_secret,
                                 sharenums, allocated_size,
@@ -677,7 +751,7 @@ class StorageServer(service.MultiService, Referenceable):
         # to a particular owner.
         alreadygot = set()
         bucketwriters = {} # k: shnum, v: BucketWriter
-        si_s = idlib.b2a(storage_index)
+        si_dir = storage_index_to_dir(storage_index)
 
         # in this implementation, the lease information (including secrets)
         # goes into the share files themselves. It could also be put into a
@@ -702,17 +776,20 @@ class StorageServer(service.MultiService, Referenceable):
             sf = ShareFile(fn)
             sf.add_or_renew_lease(lease_info)
 
+        if self.readonly_storage:
+            # we won't accept new shares
+            return alreadygot, bucketwriters
+
         for shnum in sharenums:
-            incominghome = os.path.join(self.incomingdir, si_s, "%d" % shnum)
-            finalhome = os.path.join(self.sharedir, si_s, "%d" % shnum)
+            incominghome = os.path.join(self.incomingdir, si_dir, "%d" % shnum)
+            finalhome = os.path.join(self.sharedir, si_dir, "%d" % shnum)
             if os.path.exists(incominghome) or os.path.exists(finalhome):
                 # great! we already have it. easy.
                 pass
             elif no_limits or remaining_space >= space_per_bucket:
                 # ok! we need to create the new share file.
-                fileutil.make_dirs(os.path.join(self.incomingdir, si_s))
                 bw = BucketWriter(self, incominghome, finalhome,
-                                  space_per_bucket, lease_info)
+                                  space_per_bucket, lease_info, canary)
                 if self.no_storage:
                     bw.throw_out_all_data = True
                 bucketwriters[shnum] = bw
@@ -724,7 +801,7 @@ class StorageServer(service.MultiService, Referenceable):
                 pass
 
         if bucketwriters:
-            fileutil.make_dirs(os.path.join(self.sharedir, si_s))
+            fileutil.make_dirs(os.path.join(self.sharedir, si_dir))
 
         return alreadygot, bucketwriters
 
@@ -737,7 +814,7 @@ class StorageServer(service.MultiService, Referenceable):
             header = f.read(32)
             f.close()
             if header[:32] == MutableShareFile.MAGIC:
-                sf = MutableShareFile(filename)
+                sf = MutableShareFile(filename, self)
                 # note: if the share has been migrated, the renew_lease()
                 # call will throw an exception, with information to help the
                 # client update the lease.
@@ -750,7 +827,7 @@ class StorageServer(service.MultiService, Referenceable):
             raise IndexError("no such lease to renew")
 
     def remote_cancel_lease(self, storage_index, cancel_secret):
-        storagedir = os.path.join(self.sharedir, idlib.b2a(storage_index))
+        storagedir = os.path.join(self.sharedir, storage_index_to_dir(storage_index))
 
         remaining_files = 0
         total_space_freed = 0
@@ -765,7 +842,7 @@ class StorageServer(service.MultiService, Referenceable):
             header = f.read(32)
             f.close()
             if header[:32] == MutableShareFile.MAGIC:
-                sf = MutableShareFile(filename)
+                sf = MutableShareFile(filename, self)
                 # note: if the share has been migrated, the renew_lease()
                 # call will throw an exception, with information to help the
                 # client update the lease.
@@ -787,18 +864,22 @@ class StorageServer(service.MultiService, Referenceable):
         if not remaining_files:
             fileutil.rm_dir(storagedir)
         self.consumed -= total_space_freed
+        if self.stats_provider:
+            self.stats_provider.count('storage_server.bytes_freed', total_space_freed)
         if not found_buckets:
             raise IndexError("no such lease to cancel")
 
     def bucket_writer_closed(self, bw, consumed_size):
         self.consumed += consumed_size
+        if self.stats_provider:
+            self.stats_provider.count('storage_server.bytes_added', consumed_size)
         del self._active_writers[bw]
 
     def _get_bucket_shares(self, storage_index):
         """Return a list of (shnum, pathname) tuples for files that hold
         shares for this storage_index. In each tuple, 'shnum' will always be
         the integer form of the last component of 'pathname'."""
-        storagedir = os.path.join(self.sharedir, idlib.b2a(storage_index))
+        storagedir = os.path.join(self.sharedir, storage_index_to_dir(storage_index))
         try:
             for f in os.listdir(storagedir):
                 if NUM_RE.match(f):
@@ -809,7 +890,7 @@ class StorageServer(service.MultiService, Referenceable):
             pass
 
     def _get_incoming_shares(self, storage_index):
-        incomingdir = os.path.join(self.incomingdir, idlib.b2a(storage_index))
+        incomingdir = os.path.join(self.incomingdir, storage_index_to_dir(storage_index))
         try:
             for f in os.listdir(incomingdir):
                 if NUM_RE.match(f):
@@ -845,10 +926,11 @@ class StorageServer(service.MultiService, Referenceable):
                                                secrets,
                                                test_and_write_vectors,
                                                read_vector):
-        si_s = idlib.b2a(storage_index)
+        si_s = si_b2a(storage_index)
+        si_dir = storage_index_to_dir(storage_index)
         (write_enabler, renew_secret, cancel_secret) = secrets
         # shares exist if there is a file for them
-        bucketdir = os.path.join(self.sharedir, si_s)
+        bucketdir = os.path.join(self.sharedir, si_dir)
         shares = {}
         if os.path.isdir(bucketdir):
             for sharenum_s in os.listdir(bucketdir):
@@ -857,8 +939,8 @@ class StorageServer(service.MultiService, Referenceable):
                 except ValueError:
                     continue
                 filename = os.path.join(bucketdir, sharenum_s)
-                msf = MutableShareFile(filename)
-                msf.check_write_enabler(write_enabler)
+                msf = MutableShareFile(filename, self)
+                msf.check_write_enabler(write_enabler, si_s)
                 shares[sharenum] = msf
         # write_enabler is good for all existing shares.
 
@@ -917,13 +999,17 @@ class StorageServer(service.MultiService, Referenceable):
         my_nodeid = self.my_nodeid
         fileutil.make_dirs(bucketdir)
         filename = os.path.join(bucketdir, "%d" % sharenum)
-        share = create_mutable_sharefile(filename, my_nodeid, write_enabler)
+        share = create_mutable_sharefile(filename, my_nodeid, write_enabler,
+                                         self)
         return share
 
     def remote_slot_readv(self, storage_index, shares, readv):
-        si_s = idlib.b2a(storage_index)
+        si_s = si_b2a(storage_index)
+        lp = log.msg("storage: slot_readv %s %s" % (si_s, shares),
+                     facility="tahoe.storage", level=log.OPERATIONAL)
+        si_dir = storage_index_to_dir(storage_index)
         # shares exist if there is a file for them
-        bucketdir = os.path.join(self.sharedir, si_s)
+        bucketdir = os.path.join(self.sharedir, si_dir)
         if not os.path.isdir(bucketdir):
             return {}
         datavs = {}
@@ -934,8 +1020,10 @@ class StorageServer(service.MultiService, Referenceable):
                 continue
             if sharenum in shares or not shares:
                 filename = os.path.join(bucketdir, sharenum_s)
-                msf = MutableShareFile(filename)
+                msf = MutableShareFile(filename, self)
                 datavs[sharenum] = msf.readv(readv)
+        log.msg("returning shares %s" % (datavs.keys(),),
+                facility="tahoe.storage", level=log.NOISY, parent=lp)
         return datavs
 
 
@@ -972,18 +1060,19 @@ section starts. Each offset is measured from the beginning of the file.
 def allocated_size(data_size, num_segments, num_share_hashes,
                    uri_extension_size):
     wbp = WriteBucketProxy(None, data_size, 0, num_segments, num_share_hashes,
-                           uri_extension_size)
+                           uri_extension_size, None)
     uri_extension_starts_at = wbp._offsets['uri_extension']
     return uri_extension_starts_at + 4 + uri_extension_size
 
 class WriteBucketProxy:
     implements(IStorageBucketWriter)
     def __init__(self, rref, data_size, segment_size, num_segments,
-                 num_share_hashes, uri_extension_size):
+                 num_share_hashes, uri_extension_size, nodeid):
         self._rref = rref
         self._data_size = data_size
         self._segment_size = segment_size
         self._num_segments = num_segments
+        self._nodeid = nodeid
 
         effective_segments = mathutil.next_power_of_k(num_segments,2)
         self._segment_hash_size = (2*effective_segments - 1) * HASH_SIZE
@@ -1021,6 +1110,13 @@ class WriteBucketProxy:
         assert len(offset_data) == 0x24
         self._offset_data = offset_data
 
+    def __repr__(self):
+        if self._nodeid:
+            nodeid_s = idlib.nodeid_b2a(self._nodeid)
+        else:
+            nodeid_s = "[None]"
+        return "<allmydata.storage.WriteBucketProxy for node %s>" % nodeid_s
+
     def start(self):
         return self._write(0, self._offset_data)
 
@@ -1100,6 +1196,9 @@ class WriteBucketProxy:
     def close(self):
         return self._rref.callRemote("close")
 
+    def abort(self):
+        return self._rref.callRemote("abort")
+
 class ReadBucketProxy:
     implements(IStorageBucketReader)
     def __init__(self, rref):
