@@ -3,6 +3,7 @@ from zope.interface import implements
 from twisted.trial import unittest
 from twisted.internet import defer
 from twisted.python.failure import Failure
+from foolscap import eventual
 from allmydata import encode, upload, download, hashtree, uri
 from allmydata.util import hashutil
 from allmydata.util.assertutil import _assert
@@ -33,9 +34,15 @@ class FakeBucketWriterProxy:
     def startIfNecessary(self):
         return defer.succeed(self)
     def start(self):
+        if self.mode == "lost-early":
+            f = Failure(LostPeerError("I went away early"))
+            return eventual.fireEventually(f)
         return defer.succeed(self)
 
     def put_block(self, segmentnum, data):
+        if self.mode == "lost-early":
+            f = Failure(LostPeerError("I went away early"))
+            return eventual.fireEventually(f)
         def _try():
             assert not self.closed
             assert segmentnum not in self.blocks
@@ -84,6 +91,9 @@ class FakeBucketWriterProxy:
             self.closed = True
         return defer.maybeDeferred(_try)
 
+    def abort(self):
+        return defer.succeed(None)
+
     def get_block(self, blocknum):
         def _try():
             assert isinstance(blocknum, (int, long))
@@ -154,9 +164,12 @@ class Encode(unittest.TestCase):
                   expected_block_hashes, expected_share_hashes):
         data = make_data(datalen)
         # force use of multiple segments
-        options = {"max_segment_size": max_segment_size, 'needed_and_happy_and_total_shares': (25, 75, 100)}
-        e = encode.Encoder(options)
+        e = encode.Encoder()
         u = upload.Data(data)
+        u.max_segment_size = max_segment_size
+        u.encoding_param_k = 25
+        u.encoding_param_happy = 75
+        u.encoding_param_n = 100
         eu = upload.EncryptAnUploadable(u)
         d = e.set_encrypted_uploadable(eu)
 
@@ -282,14 +295,17 @@ class Roundtrip(unittest.TestCase):
 
     def send(self, k_and_happy_and_n, AVAILABLE_SHARES, max_segment_size,
              bucket_modes, data):
+        k, happy, n = k_and_happy_and_n
         NUM_SHARES = k_and_happy_and_n[2]
         if AVAILABLE_SHARES is None:
             AVAILABLE_SHARES = NUM_SHARES
-        # force use of multiple segments
-        options = {"max_segment_size": max_segment_size,
-                   "needed_and_happy_and_total_shares": k_and_happy_and_n}
-        e = encode.Encoder(options)
+        e = encode.Encoder()
         u = upload.Data(data)
+        # force use of multiple segments by using a low max_segment_size
+        u.max_segment_size = max_segment_size
+        u.encoding_param_k = k
+        u.encoding_param_happy = happy
+        u.encoding_param_n = n
         eu = upload.EncryptAnUploadable(u)
         d = e.set_encrypted_uploadable(eu)
 
@@ -613,6 +629,15 @@ class Roundtrip(unittest.TestCase):
                        [(i, "lost") for i in range(9, 10)])
         return self.send_and_recover((4,8,10), bucket_modes=modemap)
 
+    def test_lost_one_shareholder_early(self):
+        # we have enough shareholders when we choose peers, but just before
+        # we send the 'start' message, we lose one of them. The upload should
+        # still succeed, as long as we still have 'shares_of_happiness' peers
+        # left.
+        modemap = dict([(i, "good") for i in range(9)] +
+                       [(i, "lost-early") for i in range(9, 10)])
+        return self.send_and_recover((4,8,10), bucket_modes=modemap)
+
     def test_lost_many_shareholders(self):
         # we have enough shareholders when we start, but one segment in we
         # lose all but one of them. The upload should fail.
@@ -621,7 +646,7 @@ class Roundtrip(unittest.TestCase):
         d = self.send_and_recover((4,8,10), bucket_modes=modemap)
         def _done(res):
             self.failUnless(isinstance(res, Failure))
-            self.failUnless(res.check(encode.NotEnoughPeersError))
+            self.failUnless(res.check(encode.NotEnoughPeersError), res)
         d.addBoth(_done)
         return d
 
