@@ -1,16 +1,22 @@
 
 from twisted.trial import unittest
 
-from twisted.application import service
 from twisted.internet import defer
-from foolscap import Referenceable
 import time, os.path, stat
 import itertools
 from allmydata import interfaces
-from allmydata.util import fileutil, hashutil, idlib
+from allmydata.util import fileutil, hashutil
 from allmydata.storage import BucketWriter, BucketReader, \
-     WriteBucketProxy, ReadBucketProxy, StorageServer, MutableShareFile
+     WriteBucketProxy, ReadBucketProxy, StorageServer, MutableShareFile, \
+     storage_index_to_dir
 from allmydata.interfaces import BadWriteEnablerError
+from allmydata.test.common import LoggingServiceParent
+
+class FakeCanary:
+    def notifyOnDisconnect(self, *args, **kwargs):
+        pass
+    def dontNotifyOnDisconnect(self, marker):
+        pass
 
 class Bucket(unittest.TestCase):
     def make_workdir(self, name):
@@ -33,7 +39,8 @@ class Bucket(unittest.TestCase):
 
     def test_create(self):
         incoming, final = self.make_workdir("test_create")
-        bw = BucketWriter(self, incoming, final, 200, self.make_lease())
+        bw = BucketWriter(self, incoming, final, 200, self.make_lease(),
+                          FakeCanary())
         bw.remote_write(0, "a"*25)
         bw.remote_write(25, "b"*25)
         bw.remote_write(50, "c"*25)
@@ -42,14 +49,15 @@ class Bucket(unittest.TestCase):
 
     def test_readwrite(self):
         incoming, final = self.make_workdir("test_readwrite")
-        bw = BucketWriter(self, incoming, final, 200, self.make_lease())
+        bw = BucketWriter(self, incoming, final, 200, self.make_lease(),
+                          FakeCanary())
         bw.remote_write(0, "a"*25)
         bw.remote_write(25, "b"*25)
         bw.remote_write(50, "c"*7) # last block may be short
         bw.remote_close()
 
         # now read from it
-        br = BucketReader(final)
+        br = BucketReader(bw.finalhome)
         self.failUnlessEqual(br.remote_read(0, 25), "a"*25)
         self.failUnlessEqual(br.remote_read(25, 25), "b"*25)
         self.failUnlessEqual(br.remote_read(50, 7), "c"*7)
@@ -69,7 +77,8 @@ class BucketProxy(unittest.TestCase):
         final = os.path.join(basedir, "bucket")
         fileutil.make_dirs(basedir)
         fileutil.make_dirs(os.path.join(basedir, "tmp"))
-        bw = BucketWriter(self, incoming, final, size, self.make_lease())
+        bw = BucketWriter(self, incoming, final, size, self.make_lease(),
+                          FakeCanary())
         rb = RemoteBucket()
         rb.target = bw
         return bw, rb, final
@@ -85,13 +94,13 @@ class BucketProxy(unittest.TestCase):
         pass
 
     def test_create(self):
-        bw, rb, final = self.make_bucket("test_create", 500)
+        bw, rb, sharefname = self.make_bucket("test_create", 500)
         bp = WriteBucketProxy(rb,
                               data_size=300,
                               segment_size=10,
                               num_segments=5,
                               num_share_hashes=3,
-                              uri_extension_size=500)
+                              uri_extension_size=500, nodeid=None)
         self.failUnless(interfaces.IStorageBucketWriter.providedBy(bp))
 
     def test_readwrite(self):
@@ -115,13 +124,14 @@ class BucketProxy(unittest.TestCase):
                         for i in (1,9,13)]
         uri_extension = "s" + "E"*498 + "e"
 
-        bw, rb, final = self.make_bucket("test_readwrite", 1414)
+        bw, rb, sharefname = self.make_bucket("test_readwrite", 1414)
         bp = WriteBucketProxy(rb,
                               data_size=95,
                               segment_size=25,
                               num_segments=4,
                               num_share_hashes=3,
-                              uri_extension_size=len(uri_extension))
+                              uri_extension_size=len(uri_extension),
+                              nodeid=None)
 
         d = bp.start()
         d.addCallback(lambda res: bp.put_block(0, "a"*25))
@@ -137,7 +147,7 @@ class BucketProxy(unittest.TestCase):
 
         # now read everything back
         def _start_reading(res):
-            br = BucketReader(final)
+            br = BucketReader(sharefname)
             rb = RemoteBucket()
             rb.target = br
             rbp = ReadBucketProxy(rb)
@@ -178,7 +188,7 @@ class BucketProxy(unittest.TestCase):
 class Server(unittest.TestCase):
 
     def setUp(self):
-        self.sparent = service.MultiService()
+        self.sparent = LoggingServiceParent()
         self._lease_secret = itertools.count()
     def tearDown(self):
         return self.sparent.stopService()
@@ -201,7 +211,33 @@ class Server(unittest.TestCase):
         cancel_secret = hashutil.tagged_hash("blah", "%d" % self._lease_secret.next())
         return ss.remote_allocate_buckets(storage_index,
                                           renew_secret, cancel_secret,
-                                          sharenums, size, Referenceable())
+                                          sharenums, size, FakeCanary())
+
+    def test_dont_overfill_dirs(self):
+        """
+        This test asserts that if you add a second share whose storage index
+        share lots of leading bits with an extant share (but isn't the exact
+        same storage index), this won't add an entry to the share directory.
+        """
+        ss = self.create("test_dont_overfill_dirs")
+        already, writers = self.allocate(ss, "storageindex", [0], 10)
+        for i, wb in writers.items():
+            wb.remote_write(0, "%10d" % i)
+            wb.remote_close()
+        storedir = os.path.join(self.workdir("test_dont_overfill_dirs"),
+                                "shares")
+        children_of_storedir = set(os.listdir(storedir))
+
+        # Now store another one under another storageindex that has leading
+        # chars the same as the first storageindex.
+        already, writers = self.allocate(ss, "storageindey", [0], 10)
+        for i, wb in writers.items():
+            wb.remote_write(0, "%10d" % i)
+            wb.remote_close()
+        storedir = os.path.join(self.workdir("test_dont_overfill_dirs"),
+                                "shares")
+        new_children_of_storedir = set(os.listdir(storedir))
+        self.failUnlessEqual(children_of_storedir, new_children_of_storedir)
 
     def test_remove_incoming(self):
         ss = self.create("test_remove_incoming")
@@ -209,17 +245,15 @@ class Server(unittest.TestCase):
         for i,wb in writers.items():
             wb.remote_write(0, "%10d" % i)
             wb.remote_close()
-        incomingdir = os.path.join(self.workdir("test_remove_incoming"),
-                                   "shares", "incoming")
-        leftover_dirs = os.listdir(incomingdir)
-        self.failUnlessEqual(leftover_dirs, [])
+        incomingdir = os.path.dirname(os.path.dirname(os.path.dirname(wb.incominghome)))
+        self.failIf(os.path.exists(incomingdir))
 
     def test_allocate(self):
         ss = self.create("test_allocate")
 
         self.failUnlessEqual(ss.remote_get_buckets("vid"), {})
 
-        canary = Referenceable()
+        canary = FakeCanary()
         already,writers = self.allocate(ss, "vid", [0,1,2], 75)
         self.failUnlessEqual(already, set())
         self.failUnlessEqual(set(writers.keys()), set([0,1,2]))
@@ -253,7 +287,7 @@ class Server(unittest.TestCase):
 
     def test_sizelimits(self):
         ss = self.create("test_sizelimits", 5000)
-        canary = Referenceable()
+        canary = FakeCanary()
         # a newly created and filled share incurs this much overhead, beyond
         # the size we request.
         OVERHEAD = 3*4
@@ -336,7 +370,7 @@ class Server(unittest.TestCase):
 
     def test_leases(self):
         ss = self.create("test_leases")
-        canary = Referenceable()
+        canary = FakeCanary()
         sharenums = range(5)
         size = 100
 
@@ -444,7 +478,7 @@ class Server(unittest.TestCase):
 class MutableServer(unittest.TestCase):
 
     def setUp(self):
-        self.sparent = service.MultiService()
+        self.sparent = LoggingServiceParent()
         self._lease_secret = itertools.count()
     def tearDown(self):
         return self.sparent.stopService()
@@ -520,9 +554,10 @@ class MutableServer(unittest.TestCase):
         #self.failUnlessEqual(s0.remote_get_length(), 100)
 
         bad_secrets = ("bad write enabler", secrets[1], secrets[2])
-        self.failUnlessRaises(BadWriteEnablerError,
-                              write, "si1", bad_secrets,
-                              {}, [])
+        f = self.failUnlessRaises(BadWriteEnablerError,
+                                  write, "si1", bad_secrets,
+                                  {}, [])
+        self.failUnless("The write enabler was recorded by nodeid 'aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa'." in f, f)
 
         # this testv should fail
         answer = write("si1", secrets,
@@ -776,7 +811,7 @@ class MutableServer(unittest.TestCase):
         # create a random non-numeric file in the bucket directory, to
         # exercise the code that's supposed to ignore those.
         bucket_dir = os.path.join(self.workdir("test_leases"),
-                                  "shares", idlib.b2a("si1"))
+                                  "shares", storage_index_to_dir("si1"))
         f = open(os.path.join(bucket_dir, "ignore_me.txt"), "w")
         f.write("you ought to be ignoring me\n")
         f.close()
